<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Exception;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Municipio;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Fornecedor;
use Geral\Entities\Tipobaixapag;
use Fiscal\Entities\Notafiscal;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabBanrisulService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabBanrisulService extends GeraCnabService{

    const NUM_LOTE_HEADER_ARQUIVO = '0000';
    const ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    const ID_TP_ARQUIVO_HEADER_LOTE = '1';
    const ID_TP_ARQUIVO_SEGMENTO = '3';
    const ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    const TIPO_INS_JUR_CPF = 1;
    const TIPO_INS_JUR_CNPJ = 2;
    const NOME_BANCO = 'Banrisul';
    const CDBANCO = '041';
    const REMESSA = 1;
    const RETORNO = 2;
    const VERSAO_LAYOUT = '040';
    const TIPO_OPERACAO_CREDITO = 'C';
    const TIPO_SERVICO = '20';
    const TIPO_SERVICO_TRIBUTO = '22';
    const VERSAO_LOTE = '031';
    const VERSAO_HEADER_LOTE = '040';
    const VERSAO_LOTE_TITULO_BANC = '030';
    const VERSAO_LOTE_TITULO_TRIBUTO = '010';
    const SEGMENTO_A = 'A';
    const EXCLUSAO = 'E';
    const INCLUSAO = 'I';
    const BLOQUEIO = 'B';
    const MOEDA = 'BRL';
    const SEGMETO_B = 'B';
    const LOTE_SERVICO_TRAILER_ARQUIVO = '9999';
    const ID_TP_ARQUIVO_TRAILER = '9';
    const SEGMENTO_J = 'J';
    const SEGMENTO_J52 = '52';
    const SEGMENTO_O = 'O';
    const SEGMETO_I = 'I';
    const SEGMETO_D = 'D';
    const SEGMENTO_N = 'N';
    const TIPO_BAIXA_PIX = 'F';
    const TIPO_BAIXA_PIX_QRCODE = 'Z';
    const TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL = 'N';
    const TIPO_BAIXA_PAG_IPVA = 'V';
    const TIPO_BAIXA_PAG_GARE = 'X';
    const TIPO_BAIXA_PAG_DARJ = 'J';
    const TIPO_BAIXA_DAE = 'Q';
    const TIPO_BAIXA_ISS = 'N';

    private $num_header_lote = 0;
    private $seqRegLote = 0;
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $vrPagtLote = 0.00; // Somatório dos valores de inclusao do lote;
    private $formapgto;

    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){
        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->filial->getNrinsjurfili();
                $this->nomeempresa = $this->filial->getNmfilial();
                $this->nomerazempresa = $this->filial->getNmrazsocfili();
            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->empresa->getNrinsjurempr();
                $this->nomeempresa = $this->empresa->getNmempresa();
                $this->nomerazempresa = $this->empresa->getNmrazsocempr();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null){
        try{

            $file = fopen($caminhoArquivo.'.REM', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);

            //Percorre as formas de pagamento
            foreach($boletosAgrup as $segmento => $boletosFormaPgo){
                
                if($segmento == 'SEGMENTOAB'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);

                        foreach($boletosBco as $boleto){
                            $segmentoA = self::geraSegmentoA($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoA);
                            $segmentoB = self::geraSegmentoB($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoB);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                } 
                if($segmento == 'SEGMENTOJ'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLote($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ($boleto);
                            fprintf($file, "%s",$segmentoJ);
                            /** @var $boleto Boletpagar */
                            $segmentoJ = self::geraSegmentoJ52($boleto);
                            fprintf($file, "%s",$segmentoJ);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTON'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLote($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoN = self::geraSegmentoN($boleto);
                            fprintf($file, "%s",$segmentoN);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTOO'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLote($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoO = self::geraSegmentoO($boleto);
                            fprintf($file, "%s",$segmentoO);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTOAPIX'){
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        //Percorre os Boletos
                        foreach($boletosBco as $boleto){
                             $headerlote  = self::geraHeaderLote($key, $boletosBco);
                             fprintf($file, "%s",$headerlote);
                            /** @var $boleto Boletpagar */
                            $segmentoA = self::geraSegmentoAPIX($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoA);
                            $segmentoB = self::geraSegmentoBPIX($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoB);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
            }

            $trailerArquivo = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailerArquivo);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /** Retorna array de Boletos Agrupados por Forma de Pagamento e por código do banco favorecido*/
    private function agrupaBoletos($boletos){

        $boletosAgroup = array();
        $segmentosAB = array(Tipobaixapag::TIPOBAIXA_CNAB,
            Tipobaixapag::TIPOBAIXA_CNAB_DOC,
            Tipobaixapag::TIPOBAIXA_TED_CIP,
            Tipobaixapag::TIPO_BAIXA_TED_STR,
        );

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

        $segmentoO = array(
            Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, 
            Tipobaixapag::TIPO_BAIXA_FGTS, 
            self::TIPO_BAIXA_DAE, 
            self::TIPO_BAIXA_ISS,
            self::TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL
        );
        
        $segmentoN = array(
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES,
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL,
            Tipobaixapag::TIPO_BAIXA_GPS,
            self::TIPO_BAIXA_PAG_IPVA,
            self::TIPO_BAIXA_PAG_GARE,
            self::TIPO_BAIXA_PAG_DARJ
        );
        
        $segmentoAPIX = array('F');
        
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){

            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag(
                $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
            );

            /** @var Tipobaixapag $tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByNrorgAndCdtpbaixapag($boleto->getNrorg(), $boleto->getCdtpbaixapag());
            
            if($tipobaixapag)
                $idbaixautilp = $tipobaixapag->getIdbaixautilp();
            else $idbaixautilp = null;

            if(in_array($idbaixautilp,$segmentosAB)){

                if(self::getBanco($boleto) == GeraCnabService::CODIGO_COMPENSACAO_BANRISUL)
                    $boletosAgroup['SEGMENTOAB'] [GeraCnabService::CODIGO_COMPENSACAO_BANRISUL] [] = $boleto;
                else
                    $boletosAgroup['SEGMENTOAB'] ['OUTROS_BANCOS'] [] = $boleto;

            }else{
                if(in_array($idbaixautilp,$segmentoJ)){
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BANRISUL)){
                        $boletosAgroup['SEGMENTOJ'] [GeraCnabService::CODIGO_COMPENSACAO_BANRISUL] [] = $boleto;
                    }
                    else{
                        $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                    }

                }else{
                    if(in_array($idbaixautilp,$segmentoO)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BANRISUL)){
                            $boletosAgroup['SEGMENTOO'] [GeraCnabService::CODIGO_COMPENSACAO_BANRISUL] [] = $boleto;
                        }
                        else{
                            $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;
                        }

                    } if(in_array($idbaixautilp,$segmentoAPIX)){
                        $cdbanco = self::getBanco($boleto, 'F');
    
                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BANRISUL))
                        {
                            $boletosAgroup['SEGMENTOAPIX'] [GeraCnabService::CODIGO_COMPENSACAO_BANRISUL] [] = $boleto;
                        }
                        else
                        {
                            $boletosAgroup['SEGMENTOAPIX'] ['OUTROS_BANCOS'] [] = $boleto;
                        }
    
                    }else{
                        if(in_array($idbaixautilp,$segmentoN)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BANRISUL)){
                            $boletosAgroup['SEGMENTON'] [GeraCnabService::CODIGO_COMPENSACAO_BANRISUL] [] = $boleto;
                        }
                        else{
                            $boletosAgroup['SEGMENTON'] ['OUTROS_BANCOS'] [] = $boleto;
                        }

                        }else{
                            throw new \Exception ('Operação bloqueada. Tipo de Baixa não reconhecido.');
                        }
                    }
                }
            }
        }
        return $boletosAgroup;
    }


    // Registro Header de Arquivo
    /**
     * @throws Exception
     * @return string
     */
    public function geraHeaderArquivo(){
        try {

            $this->qtdRegiArq = 0;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação
            // CONTEUDO: Constante - 041
            // POSIÇÃO: 1 a 3
            $linha = self::CDBANCO;
            
            // DESCRIÇÃO: Lote de serviço (somente para header de arquivo)
            // CONTEUDO: Constante -0000
            // POSIÇÃO: 4 a 7
            $linha.= self::NUM_LOTE_HEADER_ARQUIVO;

            // DESCRIÇÃO: Header de arquivo - Identifica o tipo de registro.
            // CONTEUDO: Constante - 0
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_HEADER_ARQUIVO;

            // DESCRIÇÃO: Uso exclusivo Febraban/Cnab.
            // CONTEUDO: Branco
            // POSIÇÃO: 9 a 17
            $linha.= self::preenche_brancos('',9);
            
            // DESCRIÇÃO: Tipo de Inscrição da Empresa.
            // CONTEUDO: 1 = CPF 2 = CNPJ.
            // POSIÇÃO: 18 a 18
            $linha.= $this->tipoinscricao;
            
            // DESCRIÇÃO: Número da inscrição da Empresa.
            // CONTEUDO: (CPF/CNPJ).
            // POSIÇÃO: 19 a 32
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Código do Convênio no Banco.
            // CONTEUDO: informado pela Agência.
            // POSIÇÃO: 33 a 37
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),5);
            }else{
                $linha.= str_repeat('0',5);
            }
            
            // CONTEUDO: Brancos.
            // POSIÇÃO: 38 a 52
            $linha.= self::preenche_brancos('',15);
            
            // DESCRIÇÃO: Agência mantenedora da Conta.
            // POSIÇÃO: 53 a 57
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
             
            if($digitoAgencia){
                $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            } else {
                $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            }

            // DESCRIÇÃO: Constante - 0 (zero)
            // POSIÇÃO: 58 a 58
            $linha.= str_repeat('0',1);

            // CONTEUDO: 000 – (zeros) ou brancos.
            // POSIÇÃO: 59 a 61
            $linha.= '000';

            // DESCRIÇÃO: Número da Conta Corrente.
            // POSIÇÃO: 62 a 71
            $linha.= str_pad(Util::campoCnabValido(self::getContaCorrenteSemDigito() . self::getDigitoContaCorrente()), 10,'0',STR_PAD_LEFT);

            // CONTEUDO: 0 – (zero) ou branco.
            // POSIÇÃO: 72 a 72
            $linha.= str_repeat('0',1);

            // DESCRIÇÃO: Nome da Empresa.
            // POSIÇÃO: 73 a 102
            $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
            
            // DESCRIÇÃO: Nome do Banco.
            // POSIÇÃO: 103 a 132
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $linha.= self::preenche_brancos(Util::campoCnabValido($banco->getNmbanco()),30);

            // DESCRIÇÃO: uso exclusivo Febraban/Cnab
            // CONTEUDO: Brancos
            // POSIÇÃO: 133 a 142
            $linha.= str_repeat(' ',10);
            
            // DESCRIÇÃO: Código Remessa/Retorno.
            // CONTEUDO: 1 - Remessa 2 - Retorno 3 - Duplo (arquivo retorno)/ou erro (todo arquivo rejeitado)
            // POSIÇÃO: 143 a 143
            $linha.= self::REMESSA;

            // DESCRIÇÃO: Data de geração do Arquivo
            // CONTEUDO: (DDMMAAAA)
            // POSIÇÃO: 144 a 151
            $linha.= self::getDataGeracao();

            // DESCRIÇÃO: Hora de geração do Arquivo
            // CONTEUDO: (HHMMSS)
            // POSIÇÃO: 152 a 157
            $linha.= self::getHorarioAtual();
            
            // DESCRIÇÃO: Número Sequencial do Arquivo
            // CONTEUDO: Evoluir um número no sequencial a cada header de arquivo e não pode ser repetido no dia.
            // POSIÇÃO: 158 a 163
            $linha.= substr($this->remessa->getNrseqremessapag(),-6);
            
            // DESCRIÇÃO: Número da versão do leiaute do arquivo
            // CONTEUDO: 020, 030 ou 040 - constante
            // POSIÇÃO: 164 a 166
            $linha.= self::VERSAO_LAYOUT;
            
            // DESCRIÇÃO: Número Sequencial do Arquivo
            // CONTEUDO: Evoluir um número no sequencial a cada header de arquivo e não pode ser repetido no dia.
            // POSIÇÃO: 167 a 171
            $linha.= str_repeat('0',5);
            
            // DESCRIÇÃO: Para uso do Banco.
            // CONTEUDO: (Quando o campo 143 retornar com código de remessa 3, este campo conterá a mensagem informando o motivo da rejeição.
            // POSIÇÃO: 172 a 191
            $linha.= str_repeat(' ',20);
            
            // DESCRIÇÃO: Para uso da Empresa.
            // POSIÇÃO: 192 a 211
            $linha.= str_repeat(' ',20);
            
            // DESCRIÇÃO: Uso exclusivo Febraban/Cnab.
            // CONTEUDO: Brancos
            // POSIÇÃO: 212 a 240
            $linha.= str_repeat(' ',29);
            
            return $linha."\r\n";
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLote($cdbanco, $boletos = null){
        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            // DESCRIÇÃO: Código do Banco na Compensação.
            // CONTEUDO: 041 - Constante
            // POSIÇÃO: 1 a 3
            $linha = self::CDBANCO;

            // DESCRIÇÃO: Lote de Serviço.
            // CONTEUDO: Número sequencial do lote e não pode ser repetido no arquivo).
            // POSIÇÃO: 4 a 7
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Registro Header de Lote - Identifica o tipo de registro
            // CONTEUDO: 1 - Constante
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            // DESCRIÇÃO: Tipo de operação
            // CONTEUDO: C - Constante - Crédito
            // POSIÇÃO: 9 a 9
            $linha.= self::TIPO_OPERACAO_CREDITO;

            // DESCRIÇÃO: Tipo de Serviço (Indica o tipo de serviço do lote)
            // POSIÇÃO: 10 a 11
            $linha.= self::TIPO_SERVICO;
        
            // DESCRIÇÃO: Forma de Lançamento
            // CONTEUDO: 1 - Constante
            // POSIÇÃO: 12 a 13
            self::formaPagamento($cdbanco, $boletos);
            $linha.= $this->formapgto;

            // DESCRIÇÃO: Número da versão do header do lote e deve ser igual ao informado no header do arquivo.
            // CONTEUDO: 040 – Constante
            // POSIÇÃO: 14 a 16
            $linha.= self::VERSAO_HEADER_LOTE;
    
            // DESCRIÇÃO: uso exclusivo da Febraban/CNAB
            // CONTEUDO: Branco
            // POSIÇÃO: 17 a 17
            $linha.= str_repeat(' ',1);
            
            // DESCRIÇÃO: Tipo de Inscrição da Empresa.
            // CONTEUDO: 1 = CPF 2 = CNPJ.
            // POSIÇÃO: 18 a 18
            $linha.= $this->tipoinscricao;

            // DESCRIÇÃO: Número da inscrição da Empresa.
            // CONTEUDO: (CPF/CNPJ).
            // POSIÇÃO: 19 a 32
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Código do Convênio no Banco.
            // CONTEUDO: informado pela Agência.
            // POSIÇÃO: 33 a 37
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),5);
            }else{
                 $linha.= str_repeat('0',5);
            }
            
            // CONTEUDO: Brancos.
            // POSIÇÃO: 38 a 52
            $linha.= str_repeat(' ',15);
            
            // DESCRIÇÃO: Agência mantenedora da Conta.
            // POSIÇÃO: 53 a 57
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
             
            if($digitoAgencia){
                $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            } else {
                $linha.= str_pad($this->contcorr->getCdagencia(), 5, '0', STR_PAD_LEFT);
            }
            
            // CONTEUDO: 0000 – (zeros)
            // POSIÇÃO: 58 a 61
            $linha .= '0000';
            
            // DESCRIÇÃO: Número da Conta Corrente.
            // POSIÇÃO: 62 a 71
            $linha.= str_pad(Util::campoCnabValido(self::getContaCorrenteSemDigito() . self::getDigitoContaCorrente()), 10,'0',STR_PAD_LEFT);

            // CONTEUDO: Branco
            // POSIÇÃO: 72 a 72
            $linha.= str_repeat(' ',1);

            // DESCRIÇÃO: Nome da Empresa.
            // POSIÇÃO: 73 a 102
            $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
            
            // CONTEUDO: Brancos.
            // POSIÇÃO: 103 a 142
            $linha.= str_repeat(' ',40);
            
            // DESCRIÇÃO: Endereço (Nome da Rua, Av, Praça, etc...).
            // POSIÇÃO: 143 a 172
            $linha.= str_pad(substr(Util::campoCnabValido($this->empresa->getDsendeempr()),0, 30), 30);
            
            // DESCRIÇÃO: Número do local.
            // POSIÇÃO: 173 a 177
            $linha.= str_repeat('0',5);
            
            // DESCRIÇÃO: Complemento (casa, apto, sala, etc...).
            // POSIÇÃO: 178 a 192
            $linha.= str_repeat(' ',15);
            
            // DESCRIÇÃO: Nome da Cidade.
            // POSIÇÃO: 193 a 212
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
            $linha.= str_pad(substr($municipio->getNmmunicipio(), 0, 20), 20);
            
            // DESCRIÇÃO: CEP.
            // POSIÇÃO: 213 a 220
            $linha.= Util::campoCepTel($this->empresa->getNrcepempr());
            
            // DESCRIÇÃO: Sigla do Estado.
            // POSIÇÃO: 221 a 222
            $linha.= $this->empresa->getSgestado();
            
            // DESCRIÇÃO: Classificação da ordem dos registros no lote para débito:.
            // CONTEUDO: 'VA' = Ordem ascendente de valor (debita o valor menor primeiro)
            //           'VD' = Ordem descendente de valor (debita valor maior primeiro)
            //           '  ' = Não reordena os registros do lote.
            // POSIÇÃO: 223 a 224
            $linha.= str_repeat(' ',2);
            
            // DESCRIÇÃO: Uso exclusivo da Febraban/Cnab.
            // CONTEUDO: Bramcos
            // POSIÇÃO: 225 a 230
            $linha.= str_repeat(' ',6);
            
            // DESCRIÇÃO: Códigos de ocorrências p/retorno.
            // CONTEUDO: Bramcos
            // POSIÇÃO: 231 a 240
            $linha.= str_repeat(' ',10);

            return $linha."\r\n";

        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function formaPagamento($cdbanco, $boletos){
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());

            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();
        }
        
        switch ($idbaixautilp)
        {
            case 'B':
                if($cdbanco == self::CODIGO_COMPENSACAO_BANRISUL)
                    $this->formapgto = '30';
                else
                    $this->formapgto = '31';
                break;
            case 'G': $this->formapgto = '32';
                break;
            case 'H': $this->formapgto = '18';
                break;
            case 'I': $this->formapgto = '16';
                break;
            case 'O': $this->formapgto = '10';
                break;
            case 'R': $this->formapgto = '33';
                break;
            case 'U': $this->formapgto = '11';
                break;
            case 'X': $this->formapgto = '22';
                break; 
            case 'J': $this->formapgto = '21';
                break;
            case 'V': $this->formapgto = '25';
                break;
            case 'Z': $this->formapgto = '45';
                break;
            case 'F': $this->formapgto = '45';
                break;
            case 'G': $this->formapgto = '17';
                break;
            default:
                if ($cdbanco == self::CODIGO_COMPENSACAO_BANRISUL)
                     $this->formapgto = '01';
                else $this->formapgto = '03';
                break;
        }

        return $this->formapgto;
    }


    /**
     * retorna a data atual no formato ddmmYYYY
     * @return string
     */
    private function getDataGeracao(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('d').$dataatual->format('m').$dataatual->format('Y');
    }

    /**
     * @return string
     */
    private function getHorarioAtual(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('H').$dataatual->format('i').$dataatual->format('s');
    }


    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
     */
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }


    /**
     * @param \Fiscal\Entities\Boletpagar $boleto
     * @param $cdbanco
     * @throws Exception
     * @internal param $formapgto
     * @internal param array $boletos
     * @return string
     */
    public function geraSegmentoA(Boletpagar $boleto, $cdbanco){
        try {

            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação
            // CONTEUDO: Constante - 041
            // POSIÇÃO: 1 a 3
            $linha = self::CDBANCO;

            // DESCRIÇÃO: Lote de Serviço - Número do Lote no Arquivo.
            // POSIÇÃO: 4 a 7
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Registro detalhe do Lote.
            // CONTEUDO: Constante - 3
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            // DESCRIÇÃO: Número sequencial do registro no Lote
            // CONTEUDO: O primeiro registro é inicializado com 00001.
            // POSIÇÃO: 9 a 13
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Código do Segmento do Registro Detalhe.
            // CONTEUDO: Constante - A
            // POSIÇÃO: 14 a 14
            $linha.= self::SEGMENTO_A;

            // DESCRIÇÃO: Tipo de Movimento (inclusão).
            // CONTEUDO: Constante 0 (zero)
            // POSIÇÃO: 15 a 15
            $linha .= '0';
            
            // DESCRIÇÃO: Código de Instrução do movimento
            // CONTEUDO: Constante - 00 (zeros)
            // POSIÇÃO: 16 a 17
            $linha .= '00';
            
            // DESCRIÇÃO: Código da Câmara de Compensação
            // POSIÇÃO: 18 a 20
            $tipobaixapag = Tipobaixapag::findOneByCdtpbaixapagAndNrorg($boleto->getCdtpbaixapag(),$this->nrorg);
            
            if($cdbanco != "041"){
                if(floatval($boleto->getVrfinalbpag()) > 5000){
                    if($tipobaixapag->getIdbaixautilp() == "B"){
                        $linha .= '018';
                        }else {
                        $linha .= '018';
                    }
                } else {
                    $linha .= '700';
                } 
                
            } else {
                $linha .= '018';
            }

            // DESCRIÇÃO: Código do Banco do favorecido.
            // CONTEUDO: Constante - 00 (zeros)
            // POSIÇÃO: 21 a 23
            if($boleto->getCdbancbolp()){
                $linha.= str_pad(substr(trim($boleto->getCdbancbolp()),0,3),3,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_repeat('0',3);
            }
            
            // DESCRIÇÃO: Agência mantenedora da conta do favorecido..
            // POSIÇÃO: 24 a 28
             if($boleto->getCdagenbolp()){
                $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_repeat('0',5);
            }
            
            // CONTEUDO: Constante - 0 (zero) ou branco
            // POSIÇÃO: 29 a 29
            $linha.= str_repeat('0',1);
            
            // DESCRIÇÃO: Número da Conta Corrente
            // POSIÇÃO: 30 a 42
            if($this->formapgto == '10')
                $linha.= str_repeat('0',13);
            else
                $linha .= str_pad($boleto->getCdccorbolp(),13,'0', STR_PAD_LEFT);
            
            // CONTEUDO: Constante - 0 (zero) ou branco
            // POSIÇÃO: 43 a 43
            $linha.= str_repeat('0',1);
            
            // DESCRIÇÃO: Nome do Favorecido
            // POSIÇÃO: 44 a 73
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            
            // DESCRIÇÃO: Seu Número
            //  Número do documento atribuído pela Empresa
            // POSIÇÃO: 74 a 88
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),$boleto->getCdfilial(),$boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),15,' ',STR_PAD_RIGHT);
            
            // DESCRIÇÃO: Finalidade do cliente para TED ou DOC.
            // COMPOSIÇÃO: Numérico e obrigatório quando for TED.
            // POSIÇÃO: 89 a 93
            if($this->formapgto == '03')
                $linha.= '00007';
            else
                $linha.= '00005';
                
            // DESCRIÇÃO: Data do Crédito no formato
            // COMPOSIÇÃO: DDMMAAAA
            // POSIÇÃO: 94 a 101
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            // DESCRIÇÃO: Tipo de moeda
            // COMPOSIÇÃO: BRL (REAL)
            // POSIÇÃO: 102 a 104
            $linha.= self::MOEDA;
            
            // COMPOSIÇÃO: zeros
            // POSIÇÃO: 105 a 119
            $linha.= str_repeat('0',15);
            
            // DESCRIÇÃO: Valor a ser creditado.
            // POSIÇÃO: 120 a 134
            $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Número do documento atribuído pelo Banco.
            // POSIÇÃO: 135 a 154
            $linha.= str_repeat(' ',20);
            
            
            // DESCRIÇÃO: Data da efetivação do Crédito no formato DDMMAAAA.
            // COMPOSIÇÃO: É informado somente no arquivo retorno.
            // POSIÇÃO: 155 a 162
            $linha.= str_repeat(' ',8);
            
            
            // DESCRIÇÃO: Valor do Crédito efetuado.
            // COMPOSIÇÃO: É informado somente no arquivo retorno.
            // POSIÇÃO: 163 a 177
            $linha.= str_repeat(' ',15);
            
            // COMPOSIÇÃO: Brancos
            // POSIÇÃO: 178 a 182
            $linha.= str_repeat(' ',5);
            
            // DESCRIÇÃO: CIT (Código Identificador de Transferência) TED. - OPCIONAL
            // POSIÇÃO: 183 a 202
            $linha.= str_repeat(' ',20);
            
            // DESCRIÇÃO: Tipo de Inscrição
            // COMPOSIÇÃO:1 = CPF 2 = CNPJ.
            // POSIÇÃO: 203 a 203
            if ($this->empresa->getIdtpijurempr() == 'F')
                $linha.= '1'; //CPF
            else
                $linha.= '2'; //CNPJ
            
            // DESCRIÇÃO: Número de Inscrição
            // COMPOSIÇÃO: CPF ou CNPJ
            // POSIÇÃO: 204 a 217
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Uso exclusivo Febraban/Cnab
            // COMPOSIÇÃO: Brancos
            // POSIÇÃO: 218 a 229
            $linha.= str_repeat(' ',12);
            
            // COMPOSIÇÃO: 0 - não emite aviso ao favorecido.
            // POSIÇÃO: 230 a 230
            $linha.= str_repeat('0',1);
            
            // DESCRIÇÃO: Códigos das ocorrências de retorno.
            // COMPOSIÇÃO: Informado no arquivo retorno
            // POSIÇÃO: 231 a 240
            $linha.= self::preenche_brancos(' ',10);
            
            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }
    
    /**
     * * @param $nrlanctonf
     * @param $cdfilial
     * @param $nrorg
     * @return Notafiscal
     */
    private function getNotaFiscal($nrlanctonf, $cdfilial, $nrorg){
        /** @var $notafiscal Notafiscal */
        $notafiscal = Notafiscal::findOneByNrlanctonfAndCdfilialAndNrorg($nrlanctonf, $cdfilial, $nrorg);
        return $notafiscal;
    }

    /**
     * @internal param $cdbanco
     * @throws Exception
     * @return string
     * @internal param $caminho
     * @internal param $nrorg
     */
    public function geraTraillerLote($somavalores = null){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação.
            // CONTEUDO: 041 - Constante
            // POSIÇÃO: 1 a 3
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;
            
            // DESCRIÇÃO: Lote de Serviço.
            // CONTEUDO: Número sequencial do lote e não pode ser repetido no arquivo).
            // POSIÇÃO: 4 a 7
            $linha .= str_pad($this->num_header_lote, 4, '0', STR_PAD_LEFT);

            // DESCRIÇÃO: Registro Trailer de Lote - Identifica o tipo de registro
            // CONTEUDO: 5 - Constante
            // POSIÇÃO: 8 a 8
            $linha .= self::ID_TP_ARQUIVO_TRAILER_LOTE;

            // CONTEUDO: 5 - Constante
            // POSIÇÃO: 9 a 17
            $linha .= str_repeat(' ', 9);

            // DESCRIÇÃO: Total de registros no lote.
            // CONTEUDO: Soma da quantidade de registros tipos 1+3+5.
            // POSIÇÃO: 18 a 23
            $linha .= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

            // DESCRIÇÃO: Somatório dos valores.
            // CONTEUDO: (Soma dos valores dos registros tipo 3).
            // POSIÇÃO: 24 a 41
            $linha.= str_pad(str_replace(array('.',','),'', number_format($this->vrPagtLote,2)
            ),18,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: 0 (zeros) - Somatório de quantidade de moedas.
            // POSIÇÃO: 42 a 59
            $linha .= str_repeat('0', 18);

            // DESCRIÇÃO: Uso exclusivo Febraban/CNAB.
            // CONTEUDO: brancos
            // POSIÇÃO: 60 a 230
            $linha .= str_repeat(' ', 171);

            // DESCRIÇÃO: Códigos de ocorrência para retorno.
            // CONTEUDO: brancos
            // POSIÇÃO: 231 a 240
            $linha .= str_repeat(' ', 10);

            //Zera o contador de registros do lote
            $this->qtdRegiLote = 0;

            //Zera o sequencial de registros do lote
            $this->seqRegLote = 0;

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws \Exception
     */
    public function geraTrailerArquivo(){
        try { 
            
            //Quantidade de registro do arquivo
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação.
            // CONTEUDO: 041 Constante
            // POSIÇÃO: 1 a 3
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            // DESCRIÇÃO: Lote de Serviço.
            // CONTEUDO: 9999 – Constante
            // POSIÇÃO: 4 a 7
            $linha .= self::LOTE_SERVICO_TRAILER_ARQUIVO;

            // DESCRIÇÃO: Registro trailler de arquivo.
            // CONTEUDO: 9 – Constante
            // POSIÇÃO: 8 a 8
            $linha .= self::ID_TP_ARQUIVO_TRAILER;

            // DESCRIÇÃO: Uso exclusivo Febraban/CNAB
            // CONTEUDO: Brancos
            // POSIÇÃO: 9 a 17
            $linha .= str_repeat(' ', 9);
            
            // DESCRIÇÃO: Quantidade do Total de Lotes no Arquivo.
            // POSIÇÃO: 18 a 23
            $linha .= str_pad($this->qtLote, 6, '0', STR_PAD_LEFT);

            // DESCRIÇÃO: Quantidade de Registros no Arquivo
            // CONTEUDO: Soma de todos os registros do arquivo (registros tipo 0 +1+3+5+9)
            // POSIÇÃO: 24 a 29
            $linha .= str_pad($this->qtdRegiArq, 6, '0', STR_PAD_LEFT);
            
            // CONTEUDO: 000000 - (zeros)
            // POSIÇÃO: 30 a 35
            $linha .= str_repeat('0', 6);
            
            // DESCRIÇÃO: Uso exclusivo Febraban/CNAB
            // CONTEUDO: Brancos
            // POSIÇÃO: 36 a 240
            $linha .= str_repeat(' ', 205);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoJ(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação
            // CONTEUDO: Constante - 041
            // POSIÇÃO: 1 a 3
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            // DESCRIÇÃO: Lote de Serviço - Número do Lote no Arquivo.
            // POSIÇÃO: 4 a 7
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Registro detalhe do Lote.
            // CONTEUDO: Constante - 3
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            // DESCRIÇÃO: Número sequencial do registro no Lote
            // CONTEUDO: O primeiro registro é inicializado com 00001.
            // POSIÇÃO: 9 a 13
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Código do Segmento do Registro Detalhe.
            // CONTEUDO: Constante - A
            // POSIÇÃO: 14 a 14
            $linha.= self::SEGMENTO_J;

            // DESCRIÇÃO: Inclusão - Tipo de movimento.
            // CONTEUDO: 0 (zero)
            // POSIÇÃO: 15 a 15
            $linha .= '0';
            
            // DESCRIÇÃO: Código de instrução para alteração.
            // CONTEUDO: 00 (zeros)
            // POSIÇÃO: 16 a 17
            $linha .= '00';

            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            
            // DESCRIÇÃO: Código do Banco destino:
            // POSIÇÃO: 18 a 20
            $linha.= $cdbanco;
            
            // DESCRIÇÃO: Código da Moeda:
            // CONTEUDO: 9 - Real
            // POSIÇÃO: 21 a 21
            $linha.= $cdmoeda;

            // DESCRIÇÃO: DV Dígito Verificador do Código de Barras  /  DAC - Dígito de Auto Conferência.
            // POSIÇÃO: 22 a 22
            $linha.= $cddigito;

            // DESCRIÇÃO: Fator de Vencimento impresso no Código de Barras.
            // POSIÇÃO: 23 a 26
            $linha.= $valor;

            // DESCRIÇÃO: Valor impresso no Código de barras.
            // POSIÇÃO: 27 a 36
            $linha.= $cdcampo;

            // DESCRIÇÃO: Nome do Cedente.
            // POSIÇÃO: 62 a 91
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            // DESCRIÇÃO: Data do vencimento (DDMMAAAA).
            // POSIÇÃO: 92 a 99
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            // DESCRIÇÃO: Valor nominal do título.
            // POSIÇÃO: 100 a 114
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Valor do desconto + abatimento (Títulos Banrisul).
            // POSIÇÃO: 115 a 129
            $linha.= str_repeat('0',15);

            // DESCRIÇÃO: Zeros
            // POSIÇÃO: 130 a 144
            $linha.= str_repeat('0',15);

            // DESCRIÇÃO: Data do pagamento no formato DDMMAAAA.
            // POSIÇÃO: 145 a 152
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            // DESCRIÇÃO: Valor do pagamento.
            // POSIÇÃO: 153 a 167
            $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: “0” (zeros)
            // POSIÇÃO: 168 a 182
            $linha.= str_repeat('0',15);

            // DESCRIÇÃO: Brancos
            // POSIÇÃO: 183 a 187
            $linha.= str_repeat(' ',5);
            
            // DESCRIÇÃO: Para uso da Empresa (1 + CPF, 2 + CNPJ).
            // Posição: 188 a 202
            if ($this->empresa->getIdtpijurempr() == 'F')
                $linha.= '1'; //CPF
            else
                $linha.= '2'; //CNPJ
            
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);
            
            // POSIÇÃO: 203 a 217
            // Tipo: X(15)
            $linha.= str_repeat(' ',15);
            
            // DESCRIÇÃO: Uso exclusivo Febraban/Cnab. O sistema não valida. Brancos.
            // POSIÇÃO: 218 a 227
            // Tipo: X(10)
            $linha.= str_repeat(' ',10);

            // DESCRIÇÃO: DV1, DV2,DV3 – Dígitos Verificadores da Linha Digitável (só consta quando houver digitação manual).
            // POSIÇÃO: 228 a 230
            $linha.= str_repeat(' ',3);

            // DESCRIÇÃO: Códigos das Ocorrências para Retorno.
            // CONTEUDO: Informado pelo Banco no Arquivo Retorno.
            // POSIÇÃO: 231 a 240
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws \Exception
     */
    public function geraSegmentoJ52(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Campo:“041”Código do Banco na Compensação. Numérico obrigatório.
            //Posição:001 003
            //Tipo:X(3)
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            //Campo:Lote de Serviço. Número sequencial do lote. 
            // O primeiro lote do arquivo inicializa com 0001 (sequencial e não pode ser repetido no arquivo). Numérico Obrigatório.
            //Posição:004 007
            //Tipo:X(4)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Campo:“3” - Tipo de Registro. Numérico Obrigatório.
            //Posição:008 008
            //Tipo:X(1)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Campo:Número Sequencial do Registro no Lote. Numérico Obrigatório.
            //Posição:009 013
            //Tipo:X(5)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Campo:“J” - Cód. do Segmento do Registro Detalhe.
            //Posição:014 014
            //Tipo:X(1)
            $linha.= self::SEGMENTO_J;
            
            //Campo:Uso Exclusivo da FEBRABAN / CNAB. Brancos.
            //Posição:015 015
            //Tipo:X(1)
            $linha.= str_repeat(' ',1);

            //Campo:“01” - Código de Movimento Remessa.
            //Posição:016 017
            //Tipo:X(2)
            $linha.= '01';
            
            //Campo:“52” - Identificação do Registro Opcional. Numérico Obrigatório.
            //Posição:018 019
            //Tipo:X(2)
            $linha.= self::SEGMENTO_J52;
            
            //Campo:1 = CPF, 2 = CNPJ - Tipo de Inscrição do Sacado do Título.
            //Posição:020 020
            //Tipo:X(1) 
            $linha.= $this->tipoinscricao;

            //Campo:Número da Inscrição do Sacado.
            //Posição:021 035
            //Tipo:X(15)
            $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
            
            //Campo:Nome do Sacado.
            //Posição:036 075
            //Tipo:X(40)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomerazempresa)),0,40),40);

            //Campo:1 = CPF, 2 = CNPJ - Tipo de Inscrição do Cedente do Título.
            //Posição:076 076
            //Tipo:X(1)
            if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
                $linha.= self::TIPO_INS_JUR_CPF;
            }else{
                $linha.= self::TIPO_INS_JUR_CNPJ;
            }
            
            //Campo:Número da Inscrição do Cedente do título. Obrigatório para títulos com valor igual ou maior de R$250.000,00.
            //Posição:077 091
            //Tipo:X(15)
            if($boleto->getNrinscnabbop()){
                $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),15,'0', STR_PAD_LEFT);
            } else {
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);
            }
            
            //Campo:Nome do Cedente do Título
            //Posição:092 131
            //Tipo:X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
            
            //Campo:1 = CPF, 2 = CNPJ - Tipo de Inscrição do Sacador do Título.
            //Posição:132 132
            //Tipo:X(1)
            $linha.= $this->tipoinscricao;

            //Campo:Número da Inscrição do Sacador do Título.
            //Posição:133 147
            //Tipo:X(15)
            $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
            
            //Campo:Nome do Sacador do Título.
            //Posição:148 187
            //Tipo:X(40)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomerazempresa)),0,40),40);
            
            //Campo:Uso Exclusivo FEBRABAN / CNAB. Brancos.
            //Posição:188 240
            //Tipo:X(53)
            $linha.= str_repeat(' ',53);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraSegmentoI(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação
            // CONTEUDO: Constante - 041
            // POSIÇÃO: 1 a 3
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            // DESCRIÇÃO: Lote de Serviço - Número do Lote no Arquivo.
            // POSIÇÃO: 4 a 7
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Registro detalhe do Lote.
            // CONTEUDO: Constante - 3
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            // DESCRIÇÃO: Número sequencial do registro no Lote
            // CONTEUDO: O primeiro registro é inicializado com 00001.
            // POSIÇÃO: 9 a 13
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Código do Segmento do Registro Detalhe.
            // CONTEUDO: Constante - I
            // POSIÇÃO: 14 a 14
            $linha.= self::SEGMETO_I;

            // DESCRIÇÃO: Inclusão - Tipo de movimento.
            // CONTEUDO: 0 (zero)
            // POSIÇÃO: 15 a 15
            $linha.= str_repeat('0',1);
            
            // DESCRIÇÃO: Código de instrução para alteração.
            // CONTEUDO: 00 (zeros)
            // POSIÇÃO: 16 a 17
            $linha.= '00';
            
            // DESCRIÇÃO: Código de Pagamento do INSS.
            // POSIÇÃO: 18 a 21
            if($boleto->getCdreceitaguia()){
                $linha.= str_pad($boleto->getCdreceitaguia(),4,'0',STR_PAD_LEFT);
            } else {
                $linha.= str_repeat('0',4);
            }
            
            // DESCRIÇÃO: Identificador do pagamento
            // POSIÇÃO: 22 a 35
            $linha.= str_repeat('0',14);
            
            // DESCRIÇÃO: Competencia do pagamento
            // POSIÇÃO: 36 a 41
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $notafiscal = self::getNotaFiscal($contaPagar->getNrlanctonf(), $contaPagar->getCdfilial(), $contaPagar->getNrorg());
            $linha.= $notafiscal->getDtemissao()->format('mY');

            // DESCRIÇÃO: Data do vencimento. (DDMMAAAA).
            // POSIÇÃO: 42 a 49
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            // DESCRIÇÃO: Valor 06 INSS
            // POSIÇÃO: 50 a 64
            //VERIFICAR
            $linha.= str_repeat('0',15);
            
            // DESCRIÇÃO: Valor 07
            // POSIÇÃO: 65 a 79
            $linha.= str_repeat('0',15);
            
            // DESCRIÇÃO: Valor 08
            // POSIÇÃO: 80 a 94
            $linha.= str_repeat('0',15);
            
            // DESCRIÇÃO: Valor 09 - Outras entidades
            // POSIÇÃO: 95 a 109
            $linha.= str_repeat('0',15);
            
            // DESCRIÇÃO: Valor 10 Multa e Juros
            // POSIÇÃO: 110 a 124
            $linha.= str_repeat('0',15);
            
            // DESCRIÇÃO: Valor 11 Total arrecadado (valor 06 + 07 + 08+ 09 + 10 )
            // POSIÇÃO: 125 a 139
            //VERIFICAR
            $linha.= str_repeat('0',15);
            
            // COMPOSIÇÃO: BRANCO
            // POSIÇÃO: 140 a 140
            $linha.= str_repeat(' ',1);
            
            // DESCRIÇÃO: Código da Agência.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 141 a 144
            $linha.= str_repeat(' ',4);
            
            // DESCRIÇÃO: Maquina.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 145 a 148
            $linha.= str_repeat(' ',4);
            
            // DESCRIÇÃO: N.D.U. Sistema Banrisul
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 149 a 151
            $linha.= str_repeat(' ',3);
            
            // DESCRIÇÃO: N.S.U. Sistema Banrisul
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 152 a 156
            $linha.= str_repeat(' ',5);
            
            // DESCRIÇÃO: Nome do contribuinte:
            // POSIÇÃO: 157 a 186
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            
            // DESCRIÇÃO: U.S.U. Informado no arquivo retorno.
            // POSIÇÃO: 187 a 192
            $linha.= str_repeat(' ',6);
            
            // DESCRIÇÃO: Reservado
            // POSIÇÃO: 193 a 209
            $linha.= str_repeat(' ',17);
            
            // DESCRIÇÃO: Reservado para o futuro
            // POSIÇÃO: 210 a 225
            $linha.= str_repeat(' ',16);
            
            // DESCRIÇÃO: Reservado para a sequencia do contas a pagar
            // POSIÇÃO: 226 a 230
            $linha.= str_repeat(' ',5);
            
            // DESCRIÇÃO: Códigos de ocorrências p/retorno.
            // COMPOSIÇÃO: Informado no Arquivo Retorno disponibilizado pelo Banco.
            // POSIÇÃO: 231 a 240
            $linha.= str_repeat(' ',10);

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }

    }
    
     /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraSegmentoD(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação
            // CONTEUDO: Constante - 041
            // POSIÇÃO: 1 a 3
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            // DESCRIÇÃO: Lote de Serviço - Número do Lote no Arquivo.
            // POSIÇÃO: 4 a 7
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Registro detalhe do Lote.
            // CONTEUDO: Constante - 3
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            // DESCRIÇÃO: Número sequencial do registro no Lote
            // CONTEUDO: O primeiro registro é inicializado com 00001.
            // POSIÇÃO: 9 a 13
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Código do Segmento do Registro Detalhe.
            // CONTEUDO: Constante - D
            // POSIÇÃO: 14 a 14
            $linha.= self::SEGMETO_D;

            // DESCRIÇÃO: Inclusão - Tipo de movimento.
            // CONTEUDO: 0 (zero)
            // POSIÇÃO: 15 a 15
            $linha.= str_repeat('0',1);
            
            // DESCRIÇÃO: Código de instrução para alteração.
            // CONTEUDO: 00 (zeros)
            // POSIÇÃO: 16 a 17
            $linha.= '00';
            
            // DESCRIÇÃO: Codigo de barras
            // POSIÇÃO: 18 a 65
            if($boleto->getCdbarrapag()){
                $linha .= str_pad($boleto->getCdbarrapag(),48, ' ',STR_PAD_LEFT);
            } else {
               $linha.= str_repeat(' ', 48);
            }
            
            // DESCRIÇÃO: Data de pagamento
            // COMPOSIÇÃO: DDMMAAAA
            // POSIÇÃO: 66 a 73
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            // DESCRIÇÃO: Valor do pagamento
            // POSIÇÃO: 74 a 88
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Reservado
            // COMPOSIÇÃO: branco
            // POSIÇÃO: 89 a 109
            $linha.= str_repeat(' ',15);
            
            // DESCRIÇÃO: Para uso da Empresa.
            // POSIÇÃO: 110 a 169
            // COMPOSIÇÃO: Código Filial + Número de lancamento do boleto
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),$boleto->getCdfilial(),$boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),15,' ',STR_PAD_RIGHT);
            
            // DESCRIÇÃO: Código da Agência.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 170 a 173
            $linha.= str_repeat(' ',4);
            
            // DESCRIÇÃO: Maquina.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 174 a 177
            $linha.= str_repeat(' ',4);
            
            // DESCRIÇÃO: N.D.U.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 178 a 180
            $linha.= str_repeat(' ',3);
            
            // DESCRIÇÃO: N.S.U.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 181 a 186
            $linha.= str_repeat(' ',6);
            
            // DESCRIÇÃO: Nome do Cedente/favorecido.
            // COMPOSIÇÃO: Informado no Arquivo Retorno enviado pelo Banco.
            // POSIÇÃO: 187 a 216
            $linha.= str_repeat(' ',30);
            
            // DESCRIÇÃO: Reservado
            // COMPOSIÇÃO: Branco
            // POSIÇÃO: 217 a 225
            $linha.= str_repeat(' ',9);
            
            // DESCRIÇÃO: Reservado para o sequencial do contas a pagar
            // POSIÇÃO: 226 a 230
            //{ DONE : Reservado para o sequencial do contas a pagar }
            $linha.= str_repeat(' ',5);
             
            // DESCRIÇÃO: Códigos de Ocorrência p/retorno.
            // COMPOSIÇÃO: Informado no Arquivo Retorno.
            // POSIÇÃO: 231 a 240
            $linha.= str_repeat(' ',10);
      
            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    /**
     * @param Boletpagar $boleto
     * @throws Exception
     * @return string
     * @internal param $cdbanco
     */
    public function geraSegmentoN(Boletpagar $boleto){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            // DESCRIÇÃO: Código do Banco na Compensação
            // CONTEUDO: Constante - 041
            // POSIÇÃO: 1 a 3
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            // DESCRIÇÃO: Lote de Serviço - Número do Lote no Arquivo.
            // POSIÇÃO: 4 a 7
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            // DESCRIÇÃO: Registro detalhe do Lote.
            // CONTEUDO: Constante - 3
            // POSIÇÃO: 8 a 8
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            // DESCRIÇÃO: Número sequencial do registro no Lote
            // CONTEUDO: O primeiro registro é inicializado com 00001.
            // POSIÇÃO: 9 a 13
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Código do Segmento do Registro Detalhe.
            // CONTEUDO: Constante - N
            // POSIÇÃO: 14 a 14
            $linha.= 'N';
            
            // DESCRIÇÃO: Inclusão - Tipo de movimento.
            // CONTEUDO: 0 (zero)
            // POSIÇÃO: 15 a 15
            $linha.= str_repeat('0', 1);
            
            // DESCRIÇÃO: Código de instrução para alteração.
            // CONTEUDO: 00 (zeros)
            // POSIÇÃO: 16 a 17
            $linha.= str_repeat('0', 2);
            
            // DESCRIÇÃO: Número do Documento atribuído pela Empresa (Chave primaria - CDFilial + NRLANBOLPAG)
            // POSIÇÃO: 18 a 37
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),$boleto->getCdfilial(),$boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);
            
            // DESCRIÇÃO: Número do Documento atribuído pelo Banco.
            // CONTEUDO: Preencher com brancos.
            // POSIÇÃO: 38 a 57
            $linha.= str_repeat(' ', 20);
            
            // DESCRIÇÃO: Nome do Contribuinte.
            // POSIÇÃO: 58 a 87
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            
            // DESCRIÇÃO: Data de pagamento
            // CONTEUDO: DDMMAAAA
            // POSIÇÃO: 88 a 95
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            // DESCRIÇÃO: Valor do pagamento
            // CONTEUDO: DDMMAAAA
            // POSIÇÃO: 96 a 110
            $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);
            
            // Informações complementares (De acordo com o respectivo tributo)
            //POSIÇÃO: 111 a 230
            if ($this->formapgto == '16')
                $linha = self::DARFNORMAL($linha, $boleto);
            elseif($this->formapgto == '17')
                   $linha = self::GPS($linha, $boleto);
                elseif ($this->formapgto == '18')
                    $linha =self::DARFSIMPLES($linha, $boleto);
            
            // DESCRIÇÃO: Códigos das Ocorrências para Retorno.
            // CONTEUDO: Informado pelo Banco no Arquivo Retorno.
            // POSIÇÃO: 231 a 240
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function GPS($linha, $boleto){
        try {
            // DESCRIÇÃO: Receita - Código da receita do tributo 
            //CONTEUDO: Código da Receita do Tributo
            // POSIÇÃO:  111 a 116 - (X6)
            $cdreceitaguia = $boleto->getCdreceitaguia();
            if (!Empty($cdreceitaguia))
                $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 6, '0', STR_PAD_LEFT);
            else
                $linha.= str_repeat('0', 6);
           
            // DESCRIÇÃO: Tipo de identificação do contribuinte 
            // CONTEUDO: CNPJ = 2, CPF = 1 
            // POSIÇÃO: 117 a 118 - (X2)
            $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
            if ($empresa->getIdtpijurempr() == 'F')
                $linha.= '01'; //CPF
            else
                $linha.= '02'; //CNPJ
            
            // DESCRIÇÃO:  Identificação do contribuinte
            // CONTEUDO: Código identificador do contribuinte de acordo com a informação do Tipo de Identificação.
            // POSIÇÃO: 119 a 132 - (X14)
            $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
           
            // DESCRIÇÃO: Código de identificação do tributo
            // CONTEUDO: Tributos Federais ‘16’ = Tributo - DARF Normal
            //                             ‘18’ = Tributo - DARF Simples
            //                             ‘17’ = Tributo - GPS (Guia da Previdência Social) 
            // POSIÇÃO:  133 a 134 - (X2)
            $linha.= '17';
           
            // DESCRIÇÃO: Mês e ano de competência
            // CONTEUDO: (MMAAAA)
            // POSIÇÃO: 135 a 140 - (X6)
            if ($boleto->getDtapuracguia() != 0)
                $linha.= $boleto->getDtapuracguia()->format('mY');
            else
                $linha.= str_repeat('0', 6);
           
            // DESCRIÇÃO: Valor do tributo – Valor previsto do pagamento do INSS 
            // CONTEUDO: Valor de contribuição ao INSS sobre o valor do documento, expresso em moeda corrente. 
            // POSIÇÃO: 141 a 155 - (X15)
            $vrprevisto = $boleto->getVrfinalbpag() - ($boleto->getVroutrentguia() + $boleto->getVrmultabpag() + $boleto->getVrmordiabpag());
            $linha.= str_pad(str_replace(array('.',','),'', number_format($vrprevisto,2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Valor de outras entidades
            // CONTEUDO: Valor somado ao valor do documento, expresso em moeda corrente. 
            // POSIÇÃO:  156 a 170 - (X15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Atualização monetária
            // CONTEUDO: Valor da Atualização Monetária
            // POSIÇÃO:  171 a 185 - (X15)
            $vratualizacao = $boleto->getVrmultabpag() + $boleto->getVrmordiabpag();
            $linha.= str_pad(str_replace(array('.',','),'', number_format($vratualizacao,2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Uso exclusivo FEBRABAN/CNAB 
            // CONTEUDO: BRANCO
            // POSIÇÃO:  186 a 230 - (X45)
            $linha.= str_repeat(' ', 45);
            
            return $linha;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function DARFNORMAL($linha, $boleto){
        try{
            // DESCRIÇÃO: Receita - Código da receita do tributo 
            //CONTEUDO: Código da Receita do Tributo
            // POSIÇÃO:  111 a 116 - (X6)
            $cdreceitaguia = $boleto->getCdreceitaguia();
            if (!Empty($cdreceitaguia))
                $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 6, '0', STR_PAD_LEFT);
            else
                $linha.= str_repeat('0', 6);
           
            // DESCRIÇÃO: Tipo de identificação do contribuinte 
            // CONTEUDO: CNPJ = 2, CPF = 1 
            // POSIÇÃO: 117 a 118 - (X2)
            $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
            if ($empresa->getIdtpijurempr() == 'F')
                $linha.= '01'; //CPF
            else
                $linha.= '02'; //CNPJ
            
            // DESCRIÇÃO:  Identificação do contribuinte
            // CONTEUDO: Código identificador do contribuinte de acordo com a informação do Tipo de Identificação.
            // POSIÇÃO: 119 a 132 - (X14)
            $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
           
            // DESCRIÇÃO: Código de identificação do tributo
            // CONTEUDO: Tributos Federais ‘16’ = Tributo - DARF Normal
            //                             ‘18’ = Tributo - DARF Simples
            //                             ‘17’ = Tributo - GPS (Guia da Previdência Social) 
            // POSIÇÃO:  133 a 134 - (X2)
            $linha.= '16';
           
            // DESCRIÇÃO: Período de apuração
            // CONTEUDO: Dia, Mês e ano de apuração do tributo. Utilizar o formato DDMMAAAA
            // POSIÇÃO: 135 a 142 - (X8)
            if ($boleto->getDtapuracguia())
                 $linha.= $boleto->getDtapuracguia()->format('dmY');
            else $linha.= str_repeat('0', 8);
           
            // DESCRIÇÃO: Número de referência  
            // CONTEUDO: Número da guia de referencia
            // POSIÇÃO: 143 a 159 - (X17)
            $nrrefeguia = $boleto->getNrrefeguia();
            if (!$nrrefeguia)
                 $linha.= str_pad($nrrefeguia,17,'0',STR_PAD_LEFT);
            else $linha.= str_repeat('0', 17);
            
            // DESCRIÇÃO: Valor principal
            // CONTEUDO: Valor do Documento (Nominal) 
            // POSIÇÃO:  160 a 174 - (X15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Valor da multa 
            // CONTEUDO: Valor da Multa
            // POSIÇÃO:  175 a 189 - (X15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Valor dos juros / encargos 
            // CONTEUDO: Valor da Mora
            // POSIÇÃO:  190 a 204 - (X15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Data de vencimento
            // CONTEUDO: Data de vencimento nominal. Utilizar o formato DDMMAAAA.
            // POSIÇÃO:  190 a 204 - (X8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            // DESCRIÇÃO: Uso exclusivo FEBRABAN/CNAB 
            // CONTEUDO: BRANCO
            // POSIÇÃO:  213 a 230 - (X18)
            $linha.= str_repeat(' ', 18);
            
            return $linha;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function DARFSIMPLES($linha, $boleto){
        try{
            // DESCRIÇÃO: Receita - Código da receita do tributo 
            //CONTEUDO: Código da Receita do Tributo
            // POSIÇÃO:  111 a 116 - (X6)
            $cdreceitaguia = $boleto->getCdreceitaguia();
            if (!Empty($cdreceitaguia))
                $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 6, '0', STR_PAD_LEFT);
            else
                $linha.= str_repeat('0', 6);
           
            // DESCRIÇÃO: Tipo de identificação do contribuinte 
            // CONTEUDO: CNPJ = 2, CPF = 1 
            // POSIÇÃO: 117 a 118 - (X2)
            $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
            if ($empresa->getIdtpijurempr() == 'F')
                $linha.= '01'; //CPF
            else
                $linha.= '02'; //CNPJ
            
            // DESCRIÇÃO:  Identificação do contribuinte
            // CONTEUDO: Código identificador do contribuinte de acordo com a informação do Tipo de Identificação.
            // POSIÇÃO: 119 a 132 - (X14)
            $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
           
            // DESCRIÇÃO: Código de identificação do tributo
            // CONTEUDO: Tributos Federais ‘16’ = Tributo - DARF Normal
            //                             ‘18’ = Tributo - DARF Simples
            //                             ‘17’ = Tributo - GPS (Guia da Previdência Social) 
            // POSIÇÃO:  133 a 134 - (X2)
            $linha.= '18';
           
            // DESCRIÇÃO: Período de apuração
            // CONTEUDO: Dia, Mês e ano de apuração do tributo. Utilizar o formato DDMMAAAA
            // POSIÇÃO: 135 a 142 - (X8)
            if ($boleto->getDtapuracguia())
                 $linha.= $boleto->getDtapuracguia()->format('dmY');
            else $linha.= atr_repeat('0', 8);
           
            // DESCRIÇÃO: Valor da receita bruta acumulada  
            // CONTEUDO: Valor da Receita Bruta Acumulada 
            // POSIÇÃO: 143 a 157 - (X15)
            $linha.= str_pad(Util::campoCnabValido($boleto->getVroutrentguia()),15, STR_PAD_LEFT);
            
            // DESCRIÇÃO: Percentual sobre a receita bruta acumulada. 
            // CONTEUDO: Percentual sobre a Receita Bruta Acumulada 
            // POSIÇÃO:  158 a 164 - (X7)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),7,'0',STR_PAD_LEFT);
            
            // DESCRIÇÃO: Valor principal 
            // CONTEUDO: Valor principal
            // POSIÇÃO:  165 a 179 - (X15)
            if($boleto->getVrpercenttrib() != 0 && $boleto->getVroutrentguia() != 0 ){
                $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia() * $boleto->getVrpercenttrib(),2)),15,STR_PAD_LEFT);
            } else {
                 $linha .= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,STR_PAD_LEFT);
            }
            
            // DESCRIÇÃO: Valor da multa 
            // CONTEUDO: Valor da multa
            // POSIÇÃO:  180 a 194 - (X15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,STR_PAD_LEFT);
            
            // DESCRIÇÃO: Valor dos juros / encargos
            // CONTEUDO: Valor dos juros / encargos
            // POSIÇÃO:  195 a 209 - (X15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,STR_PAD_LEFT);
            
            // DESCRIÇÃO: Uso exclusivo FEBRABAN/CNAB 
            // CONTEUDO: BRANCO
            // POSIÇÃO:  210 a 230 - (X21)
            $linha.= str_repeat(' ', 21);
            
            return $linha;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function geraSegmentoB($boleto, $idopecnabpag){
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
    
            //Código do banco na compensação
            //Posição: 001 a 003
            //Picture: X(03)
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;
    
            //Lote de serviço
            //Posição: 004 a 007
            //Picture: X(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);
    
            //Tipo do registro. Constante '3'
            //Posição: 008 a 008
            //Picture: X(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;
    
            //Número sequencial do registro no lote. 
            //Posição: 009 a 013
            //Picture: X(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);
    
            //Código de segmento de registro no lote.  Constante ‘B’
            //Posição: 009 a 013
            //Picture: X(05)
            $linha.= 'B';
            
            //Uso exclusivo FEBRABAN / CNAB.
            //Posição: 015 a 017
            //Picture: X(03)
            $linha.= str_repeat(' ', 3);
    
            //Tipo de incrição do fornecedor
            //Posição: Tipo de inscrição do favorecido.
            //Picture: 9(01)
            if($boleto->getIdtpicnabbop()) {
                if ($boleto->getIdtpicnabbop() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getIdtpicnabbop() == 'j')
                        $tipoInsc =  '2'; //CNPJ
                    else
                        $tipoInsc = '9'; //Outros
                }
                $linha.= $tipoInsc;
                
                //Número de inscrição do favorecido.
                //Posição: 019 a 032
                //Picture: 9(14)
                $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
            }
            else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getForneced()->getIdtpijurforn() == 'J')
                        $tipoInsc = '2'; //CNPJ
                    else
                     $tipoInsc = '9'; //Outros
                }
                $linha.= $tipoInsc;
                //Número de inscrição do favorecido.
                //Posição: 019 a 032
                //Picture: 9(14)
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
            }
            
            //Nome da rua, avenida, praça, etc.
            //Posição: 33 a 62
            //Picture: X(30)
            $endforn = $boleto->getForneced()->getEndefornecobr();
            if(!$endforn){
                $endforn = $boleto->getForneced()->getEndeforneprinc();
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,30),30,' ',STR_PAD_RIGHT);
    
            //Número do local.
            //Posição: 63 a 67
            //Picture: 9(05)
            $linha.= str_repeat('0', 5);
            
            //Casa, apartamento, etc.
            //Posição: 68 a 82 
            //Picture: X(15)
            $linha.= str_repeat(' ', 15);
    
            //Bairro do favorecido
            //Posição: 83 a 97 
            //Picture: 9(15)
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);
    
            //Município do favorecido
            //Posição: 98 a 117
            //Picture: X(20)
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($endforn->getSgestado(), $endforn->getCdpais(), $endforn->getCdmunicipio());
            $linha.= str_pad(substr(Util::campoCnabValido($municipio->getNmmunicipio())
                ,0,20),20,' ',STR_PAD_RIGHT);
    
            //CEP
            //Posição: 118 a 122 
            //Picture: 9(05)
            if($endforn->getNrcepforn()){
                $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,5),5,'0');
            }else{
                $linha.= str_repeat('0',5);
            }
    
            //Complemento do CEP
            //Posição: 123 a 125
            //Picture: X(03)
            if(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3)){
                $linha.= substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3);
    
            }else{
                $linha.= str_repeat(' ',3);
            }
    
            //Sigla do Estado.
            //Posição: 126 a 127
            //Picture: X(02)
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);
    
            //Data do vencimento - Preenchido no formato – DDMMAAAA
            //Posição: 128 a 135
            //Picture: 9(08)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
    
            //Valor do documento (Nominal)
            //Posição: 136 a 150
            //Picture: X(15)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);
    
            //Valor do abatimento
            //Posição: 151 a 165 
            //Picture: X(15)
            $linha.= str_repeat('0', 15);
    
            //Valor do desconto
            //Posição: 166 a 180 
            //Picture: X(15)
            $linha.= str_repeat('0', 15);
    
            //Valor da mora
            //Posição: 181 a 195 
            //Picture: X(15)
            $linha.= str_repeat('0', 15);
    
            //Valor da multa
            //Posição: 196 a 210 
            //Picture: X(15)
            $linha.= str_repeat('0', 15);
    
            //Código / documento do favorecido.
            //Posição: 211 a 225 
            //Picture: X(15)
            $linha.= str_repeat(' ', 15);
    
            //Aviso ao favorecido. ‘0’ = não emite aviso.
            //Posição: 226 a 226 - 
            //Picture: X(1)
            $linha.= str_repeat('0', 1);
            
            // Uso exclusivo para o SIAPE
            //Posição: 227 a 232
            //Picture: 9(6)
            $linha.= str_repeat('0', 6);
            
            // Código do ISPB do banco destinatario (os oito primeiros dígitos do CNPJ).
            //Numéro obrigatório quando o banco não possui código de compensação.
            //Posição: 233 a 240
            //Picture: 9(8)
            $linha.= str_repeat(' ', 8);
    
            return $linha."\r\n";
        }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function geraSegmentoBPIX($boleto, $idopecnabpag){
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação. ‘041’ - Constante 
            //Posição: 1 a 3 - 
            //Picture: X(03)
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            //Lote de Serviço
            //Posição: 4 a 7
            //Picture: X(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo do Registro. ‘3’ – Constante
            //Posição: 8 a 8
            //Picture: X(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote 
            //Posição: 9 a 13 
            //Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe -‘B’ – Constante
            //Posição: 14 a 14 
            //Picture: X(01)
            $linha.= 'B';

            //Forma de iniciação
            /**
             * 01 – Chave PIX tipo Telefone
             * 02 - Chave PIX tipo E-mail
             * 03 - Chave PIX tipo CPF/CNPJ
             * 04 – Chave aleatória
             * 05 – Dados bancários
            **/
            
            //Posição: 15 a 17 -
            //Picture: X(3)
            $linha.= str_pad($boleto->getIdchavepix(), 3);

            //Tipo Inscrição do Fornecedor
            // Posicao 18 a 18
            //Picture X(1)
            $fornecedor = Fornecedor::findOneByNrorgAndCdforneced($this->nrorg, $boleto->getCdforneced());
            if ($fornecedor->getIdtpijurforn() == 'F')
                $linha.= '1';
            else
                $linha.= '2';

            //Número de Inscrição do Favorecido.
            //Posicao 19 a 32
            //Picture X(14)
            if($boleto->getNrinscnabbop()){
                $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
            }else{
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
            }
            
            // TXID - Opcional
            // Posicao 33 a 67
            // Picture X(35)
            $linha.= str_repeat(' ',35);
            
            // Tipo de conta do recebedor:
            // Posicao 68 a 127
            // Picture X(60)
            $linha.= str_repeat(' ',60);
            
            // CHAVE PIX  - obrigatorio
            // Posicao 128 a 226
            // Picture X(99)
            $linha.= str_pad($boleto->getChavepix(),99);
            
            //Uso Exclusivo para o SIAPE
            // Posicao 227 a 232
            //Picture X(6)
            $linha.= str_repeat(' ', 6);
            
            //Ocorrencias
            // Posicao 233 a 240
            //Picture X(8)
            $linha.= str_repeat(' ', 8);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    public function geraSegmentoO($boleto, $idopecnabpag){
         try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação. ‘041’ - Constante 
            //Posição: 1 a 3 - 
            //Picture: X(03)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            //Lote de Serviço
            //Posição: 4 a 7
            //Picture: X(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo do Registro. ‘3’ – Constante
            //Posição: 8 a 8
            //Picture: X(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote 
            //Posição: 9 a 13 
            //Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe -‘B’ – Constante
            //Posição: 14 a 14 
            //Picture: X(01)
            $linha.= 'B';

            //Forma de iniciação
            /**
             * 01 – Chave PIX tipo Telefone
             * 02 - Chave PIX tipo E-mail
             * 03 - Chave PIX tipo CPF/CNPJ
             * 04 – Chave aleatória
             * 05 – Dados bancários
            **/
            
            //Posição: 15 a 17 -
            //Picture: X(3)
            $linha.= $boleto->getIdchavepix();
            
            //Tipo Inscrição do Fornecedor
            // Posicao 18 a 18
            //Picture X(1)
            $fornecedor = Fornecedor::findOneByNrorgAndCdforneced($this->nrorg, $boleto->getCdforneced());
            if ($fornecedor->getIdtpijurforn() == 'F')
                $linha.= '1';
            else
                $linha.= '2';

            //Número de Inscrição do Favorecido.
            //Posicao 19 a 32
            //Picture X(14)
            if($boleto->getNrinscnabbop()){
                $linha.= str_pad(substr($boleto->getNrinscnabbop(),0,14),14,'0', STR_PAD_LEFT);
            }else{
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0', STR_PAD_LEFT);
            }
            
            // TXID - Opcional
            // Posicao 33 a 67
            // Picture X(35)
            $linha.= str_repeat(' ', 35);
            
            // Tipo de conta do recebedor:
            // Posicao 68 a 127
            // Picture X(60)
            $linha.= str_repeat(' ', 65);
            
            // CHAVE PIX  - obrigatorio
            // Posicao 128 a 226
            // Picture X(99)
            $linha.= str_pad($boleto->getChavepix(), 99);
            
            //Uso Exclusivo para o SIAPE
            // Posicao 227 a 232
            //Picture X(6)
            $linha.= str_repeat(' ', 6);
            
            //Ocorrencias
            // Posicao 233 a 240
            //Picture X(8)
            $linha.= str_repeat(' ', 8);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function geraSegmentoAPIX($boleto, $idopecnabpag){
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_BANRISUL;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'A';

            //Tipo de movimento - Posição: 015 a 015 - Picture: 9(1)
            //Código de instrução para movimento - Posição: 016 a 017 - Picture: 9(2)
            if ($idopecnabpag == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            $cdbanco   = '000';                 //PARA PIX PREENCHER COM ZEROS
            $cdmoeda   = '0';                   //PARA PIX PREENCHER COM ZEROS
            $cddigito  = '0';                   //PARA PIX PREENCHER COM ZEROS
            $fatorvenc = '0000';                //PARA PIX PREENCHER COM ZEROS
            $valor     = str_repeat('0',10);    //PARA PIX PREENCHER COM ZEROS
            $cdcampo   = str_repeat('0',25);    //PARA PIX PREENCHER COM ZEROS
            
            //Complemento de registro - Posição: 18 a 20 - Picture: 9(3)  - PIX (SPI)
            $comppix = '009';
            $linha.= $comppix;

            //Agência e Conta do Fornecedor
            //Posição: 24 a 43
            if (!$boleto->getIdchavepix()){
                if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG){
                    
                    //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                    $linha.= str_pad(substr(trim($this->contcorr->getCdbanccnab()),0,3),3,'0',STR_PAD_LEFT);
                    
                    //Número da agência mantenedora - Posição: 24 a 28 - Picture: 9(05)
                    $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
    
                    //Dídito verificador da agência - Posição: 29 a 29 - Picture: X(01)
                    $linha.= ' ';
    
                    //Número da conta - Posição: 30 a 41 - Picture: 9(12)
                    $linha.= self::preenche_brancos('',12);
    
                    //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)
                    $linha.= '0';
    
                    //Dídito verificador da agência/conta - Posição: 43 a 43 - Picture: X(01)
                    $linha.= ' ';
                }else{
                    
                    //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                    $linha.= str_pad(substr(trim($boleto->getCdbancbolp()),0,3),3,'0',STR_PAD_LEFT);
                    
                    //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
                    /** @var $bancoFornecedor Banco */
                    $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());
    
                    //{========= Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras,
                    // entrará nesse if =========}
                    if($bancoFornecedor and $bancoFornecedor->getCdpictagen()){
                        //{ se na máscara da agência NÃO tem '-' }
                        if(strpos($bancoFornecedor->getCdpictagen(),'-') === false){
                
                            //Agencia
                            $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,5)
                                ,5,'0',STR_PAD_LEFT);
                            //DV
                            $linha.= ' ';
                        }
                        else{
                            
                            //{ Caso a máscara da agência TENHA '-' }
                            //Agencia
                            $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,
                                    strpos($bancoFornecedor->getCdpictagen(),'-'))
                                ,5,'0',STR_PAD_LEFT);
    
                            //Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                            //tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                            //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                            if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,strpos($bancoFornecedor->getCdpictagen(),'-')))>0){
    
                                //DV
                                $pos = strpos($bancoFornecedor->getCdpictagen(),'-');
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,$pos,1);
                            }
                            else{
                                $linha.= ' ';
                            }
                        }
                    }
                    else{
                        //{======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema,
                        //NÃO parametrizado com as máscaras, entrará nesse else ====}
                        if(strpos($boleto->getCdagenbolp(),'-')=== false){
                            // { se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' }
                            //Agencia
                            $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);
    
                            //DV
                            $linha.= ' ';
                        }
                        else{
                            //{ caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' }
                            //Agencia
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdagenbolp(),0,
                                strpos($boleto->getCdagenbolp(),'-')
                            )),5,'0',STR_PAD_LEFT);
    
                            //DV
                            $pos = strpos($boleto->getCdagenbolp(),'-') +1;
                            $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                ,$pos,1);
                        }
                    }
    
                    //Número da conta corrente favorecido - Posição: 030 a 041 - Picture: 9(11)
                    //MAIS
                    //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)
    
                    //{========== Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado
                    //com as máscaras, entrará nesse if ===========}
                    if($bancoFornecedor and $bancoFornecedor->getCdpictcont()){
                        //{ se na máscara da conta corrente TEM '-' }
                        if(strpos($bancoFornecedor->getCdpictcont(),'-')){
    
                            //{ tamDifVer só pode assumir os valor 1 ou 2.
                            // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                            // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }
    
                            $tamDigVer = strlen(trim(substr($bancoFornecedor->getCdpictcont(),
                                strpos($bancoFornecedor->getCdpictcont(),'-')+1,
                                2)));
                            if($tamDigVer>1){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($bancoFornecedor->getCdpictcont(),'-'))),
                                    12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //Se foi cadastrada a cc do favorecido como 9999 na tela de boleto e tempos depois
                                //foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema
                                if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                        ,strpos($bancoFornecedor->getCdpictcont(),'-')))>0){
                                    //DV
                                    $pos = strpos($bancoFornecedor->getCdpictcont(),'-')+1 ;
                                    $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                        ,$pos,1);
                                }
                                else{
                                    $linha.= ' ';
                                }
    
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                    -1
                                )),12,'0',STR_PAD_LEFT);
                                
                                $linha.= ' ';
                                
                                //DV
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp())),
                                    -1);
    
                            }
    
                        }
                        else{
                            //{ CASO a máscara da conta corrente NÃO tenha '-' }
                            if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SANTANDER){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= substr($boleto->getCdccorbolp(),-1);
    
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= ' ';
                            }
                        }
    
                    }
                    // {======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com
                    //as máscaras, entrará nesse else ====}
                    else{
                        if(strpos($boleto->getCdccorbolp(),'-')){
                            //se o que foi digitado no campo de conta Corrente do favorecido da tela de boleto tem '-'
                            //CC
    
                            //{ tamDifVer só pode assumir os valor 1 ou 2.
                            // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                            // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }
    
                            $tamDigVer = strlen(trim(substr($boleto->getCdccorbolp(),
                                strpos($boleto->getCdccorbolp(),'-')+1,
                                2)));
    
                            if($tamDigVer>1){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($boleto->getCdccorbolp(),'-'))
                                ),11,'0',STR_PAD_LEFT);
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1,1));
                                    
                                $linha.= ' ';
                                
                                //DV
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+2,1));
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($boleto->getCdccorbolp(),'-'))
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1));
                            }
                        }
                        //CASO o que foi digitado no campo de conta corrente do favorecido da tela de
                        //Boleto NÃO tenha '-'
                        else{
                            if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SANTANDER){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= substr($boleto->getCdccorbolp(),-1);
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= ' ';
                            }
                        }
                    }
                }
            }else{
                //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                $linha.= '041';
                
                //Número da agência mantenedora - Posição: 24 a 43 - Picture: 9(05)
                $linha.= str_repeat(' ', 20);
            }
            
            //Nome do cedente - Posição: 44 a 73 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            
            //Cód. Refer. atribuído pelo sacado - Posição: 74 a 93 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);
            
            //Data do vencimento - Posição: 94 a 101 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Tipo da Moeda - Posição: 102 a 104 - Picture: X(3)
            $linha.= 'BRL';
            
            //Quantidade da moeda. Preencher com zeros. - Posição: 105 a 119 - Picture: X(15)
            $linha.= str_repeat('0',15);
    
            //Valor do pagamento. - Posição: 120 a 134 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Nosso número - Posição: 135 a 154 - Picture: X(20)
            $linha.= str_repeat(' ',20);
            
            //Data efetiva - Posição: 155 a 162 - Picture: 9(8)
            $linha.= str_repeat('0',8);
            
            //Valor real da efetivação do pagamento. - Posição: 163 a 177 - Picture: 9(15)
            $linha.= str_repeat('0',15);

            //Informação - Posição: 178 a 217 - Picture: X(40)
            $linha.= str_repeat(' ',40);
            
            //Código finalidade DOC - Posição: 218 a 219 - Picture: X(2)
            $linha.= str_repeat(' ',2);
            
            //Código finalidade TED - Posição: 220 a 224 - Picture: X(5)
            $linha.= str_repeat(' ',5);
            
            //Códigos finalidade complementar - Posição: 225 a 226 - Picture: X(2)
            $linha.= str_repeat(' ',2);
            
            //Uso exclusivo FEBRABAN/CNAB - Posição: 227 a 229 - Picture: X(3)
            $linha.= str_repeat(' ',3);
            
            //Zero - Posição: 230 a 230 - Picture: X(1)
            $linha.= str_repeat('0',1);

            //Código das ocorrências para retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}