<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Exception;
use Geral\Entities\Municipio;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabBrasilService
 * @package Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco
 */
class GeraCnabBrasilService extends GeraCnabService{

    const TIPO_INS_JUR_CPF = 1;
    const TIPO_INS_JUR_CNPJ = 2;
    const LOTE_SERVICO_HEADER_ARQUIVO = '0000';
    const TIPO_REGISTRO_HEADER_ARQUIVO = '0';
    const TIPO_REGISTRO_HEADER_LOTE = '1';
    const TIPO_REGISTRO_DETALHE = '3';
    const TIPO_REGISTRO_TRAILLER_LOTE ='5';
    const TIPO_REGISTRO_TRAILLER_ARQUIVO = '9';
    const ID_REMESSA = '1';
    const ID_RETORNO = '2';
    const VERSAO_LAYOUT_ARQUIVO = '030';
    const VERSAO_LAYOUT_LOTE = '020';
    const TIPO_SERVICO_PAGAMENTO_FORNECEDOR = '20';
    const TIPO_SERVICO_PAGAMENTO_DIVERSOS = '98';
    const CREDITO = 'C';
    const MOEDA = 'BRL';
    const LOTE_DE_SERVICO_TRAILLER_ARQUIVO = '9999';
    const TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL = 'N';
    const TIPO_BAIXA_PAG_IPVA = 'V';
    const TIPO_BAIXA_PAG_GARE = 'X';
    const TIPO_BAIXA_PAG_DARJ = 'J';
    const TIPO_BAIXA_PAG_DAE = 'Q';
    const TIPO_BAIXA_PIX = 'F';
    const TIPO_BAIXA_PIX_QRCODE = 'Z';

    private $formaPgto;
    private $num_header_lote = 0; // Sequencial de Header de Lote
    private $seqRegLote = 0; // Sequencial do Lote
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $num_lote = 0; // Sequencial incrementado em 1 para cada lote no arquivo
    private $vrPagtLote  = 0; //Somatório dos registros com tipo de movimento 000 (inclusão)
    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->filial->getNrinsjurfili());
                $this->nomeempresa = $this->filial->getNmrazsocfili();
                $this->dsEndeEF = $this->filial->getEndefiliprinc()->getDsendefili();
                $this->muniEF = $this->filial->getEndefiliprinc()->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->filial->getEndefiliprinc()->getNrcepfili();
                $this->estadoEF = $this->filial->getEndefiliprinc()->getSgestado();

            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->empresa->getNrinsjurempr());
                $this->nomeempresa = $this->empresa->getNmrazsocempr();
                $this->dsEndeEF = $this->empresa->getDsendeempr();
                $this->muniEF = $this->empresa->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->empresa->getNrcepempr();
                $this->estadoEF = $this->empresa->getSgestado();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws Exception
     *  //#################################################################//
        //############### REGISTRO HEADER DE ARQUIVO ######################//
        //#################################################################//
     */
    public function geraHeaderArquivo()
    {
        $linha = '';
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(3)
        $linha .= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do Lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= self::LOTE_SERVICO_HEADER_ARQUIVO;

        //Tipo de registro = Registro Header - Posição: 8 a 8 - Picture: 9(1)
        $linha .= self::TIPO_REGISTRO_HEADER_ARQUIVO;

        //Uso exclusivo FEBRABAN/CNAB -Brancos - Posição: 9 a 17 - Picture: X(9)
        $linha .= str_repeat(' ', 9);

        //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
        $linha .= $this->tipoinscricao;

        //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
        $linha .= str_pad(trim($this->inscricaojuridica), 14, '0', STR_PAD_LEFT);

        //Codigo do Convenio no Banco
        //Posição: 33 a 52
        //Picture: X(20)
        // 09 - Código do Convênio
        // 04 - Código do Produto      "0014"
        // 02 - Carteira               "00"
        // 03 - Variação da Carteira   "000"
        // 02 - Integração SAP         "Vazio"
        $identpag = self::getIdentPag();
        if ($identpag) {
            $linha .= str_pad(Util::campoCnabValido(substr(trim($identpag), 0, 9)), 9, '0', STR_PAD_LEFT) . '0126' . str_repeat(' ', 7);
        } else {
            $linha .= str_repeat('0', 9) . '0126' . str_repeat(' ', 7);
        }

        //Número da agência debitada - Posição: 53 a 57 - Picture: 9(05)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $digitoAgenciaFlag   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
        if($digitoAgenciaFlag){
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            $digitoAgencia = substr(trim($this->contcorr->getCdagencia()), -1);
        } else {
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            $digitoAgencia = ' ';
        }
        
        //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
        $linha.= $digitoAgencia;

        //Nº da C/C Debitada - Posição: 59 a 70 - Picture: 9(12)
        //Obs: A última posição da C/C é reservada para o DAC
        $digitoContaCorrenteFlag   = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
        
        if($digitoContaCorrenteFlag){
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = substr(trim($this->contcorr->getCdcontcorr()), -1);
        }else{
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = ' ';
        }

        //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
        $linha.= $digitoContaCorrente;

        //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
        $linha .= ' ';

        //Nome da empresa - Posição: 73 a 102 - Picture: X(30)
        $linha .= str_pad(substr(trim(Util::campoCnabValido($this->nomeempresa)), 0, 30), 30, ' ', STR_PAD_RIGHT);

        //Nome do Banco - Posição: 103 a 132 - Picture: X(30)
        $linha .= str_pad(substr(Util::campoCnabValido($this->contcorr->getAgencia()->getBanco()->getNmbanco()),
            0, 30), 30, ' ', STR_PAD_RIGHT);

        //Filler - Posição: 133 a 142 - Picture: X(10)
        $linha .= str_repeat(' ', 10);

        //Código remessa/Retorno (1 = Remessa, 2 = Retorno) - Posição: 143 a 143 - Picture: 9(1)
        $linha .= self::ID_REMESSA;

        //Data da geração do arquivo - Posição: 144 a 151 - Picture: 9(8)
        $dataAtual = DateUtil::getDataAtual();
        $linha .= $dataAtual->format('dmY');

        //Hora de geração do arquvio - Posição: 152 a 157 - Picture 9(6)
        $linha .= $dataAtual->format('His');

        //Número de sequência do arquivo - Posição: 158 a 163 - Picture: 9(06)
        $linha .= substr($this->remessa->getNrseqremessapag(), -6);

        //Número da versão do layout do arquivo - Posição: 164 a 166 - Picture: 9(03)
        $linha .= self::VERSAO_LAYOUT_ARQUIVO;

        //Densidade de gravação do arquivo - Posição: 167 a 171 - Picture: 9(5)
        $linha .= str_repeat('0', 5);

        // Reservado para o Banco
        // POSIÇÃO: 172 a 191
        // PICTURE: X(20)
        $linha .= str_repeat(' ', 20);

        // Reservado para a Empresa
        // POSIÇÃO: 192 a 211
        // PICTURE: X(20)
        $linha .= str_repeat(' ', 20);
        
        // // Uso Exclusivo FEBRABAN / CNAB
        // // POSIÇÃO: 212 a 240
        // // PICTURE: X(29)
        // $linha .= str_repeat(' ', 29);

        // Brancos
        // POSIÇÃO: 212 a 222
        // PICTURE: X(11)
        $linha .= str_repeat(' ', 11);

        // Conbranca s/ papel
        // POSIÇÃO: 223 a 225
        // PICTURE: X(3)
        $linha .= 'CSP';

        // Uso exclusivo das Vans
        // POSIÇÃO: 226 a 228
        // PICTURE: X(3)
        $linha .= '000';

        // POSIÇÃO: 229 a 240
        // PICTURE: X(12)
        $linha.= str_repeat(' ',12);

        return $linha."\r\n";
    }

    /**
     * @param $cdbanco
     * @internal param $formaLancamento
     * @return mixed
     * /#################################################################//
       //################ REGISTRO HEADER DE LOTE  ######################//
      //#################################################################//
     */
    public function geraHeaderLote($cdbanco, $boletos = null)
    {

        $this->seqRegLote = 0;
        $this->vrPagtLote = 0;
        $this->qtdRegiLote = 0;
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        $linha = '';
        
        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do Lote (Lote de identificação de pagtos - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad(++$this->num_lote,4,'0',STR_PAD_LEFT);

        //Tipo de registro = registro Header de lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_HEADER_LOTE;

        //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(1)
        $linha.= 'C';

        //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(2)
        $this->formaPgto = self::getFormaPagto($cdbanco, $boletos);
        if (($this->formaPgto != '30') and ($this->formaPgto != '31'))
            $linha.= self::TIPO_SERVICO_PAGAMENTO_DIVERSOS;
        else
            $linha.= self::TIPO_SERVICO_PAGAMENTO_DIVERSOS;
        

        //********************************************************
        //*********** Forma de Lançamento*************************
        //********************************************************

        //Forma de Lançamento- Posição: 12 a 13 - Picture: 9(2)
        $linha.= $this->formaPgto;

        //Nº da versão do layoute do lote - Posição: 14 a 16 - Picture: 9(3)
        if(($this->formaPgto == '30') or ($this->formaPgto == '31')){
            $linha.= '030';
        } elseif(($this->formaPgto == '01') or ($this->formaPgto == '03') or ($this->formaPgto == '10')){
             $linha.= '040';
        } else {
            $linha.= '010';
        }

        //Brancos - Posição: 17 a 17 - Picture: X(1)
        $linha.= ' ';

        //Tipo inscrição filial debitada - Posição: 18 a 18 - Picture: 9(1)
        $linha.= $this->tipoinscricao;

        //CGC filial debitada  - Posição: 19 a 32 - Picture: 9(14)
        $linha.= $this->inscricaojuridica;

        //Codigo do Convenio no Banco
        //Posição: 33 a 52
        //Picture: X(20)
        // 09 - Código do Convênio
        // 04 - Código do Produto      "0014"
        // 02 - Carteira               "00"
        // 03 - Variação da Carteira   "000"
        // 02 - Integração SAP         "Vazio"
        $identpag  = self::getIdentPag();
        if($identpag)
            $linha.= str_pad(Util::campoCnabValido(substr(trim($identpag),0,9)),9,'0', STR_PAD_LEFT). '0126'. str_repeat(' ', 7);
        else
            $linha.= str_repeat('0', 9). '0126'. str_repeat(' ', 7);

        //Número da agência debitada - Posição: 53 a 57 - Picture: 9(05)
        $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
        $digitoAgenciaFlag   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
        if($digitoAgenciaFlag){
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);
            $digitoAgencia = substr(trim($this->contcorr->getCdagencia()), -1);
        } else {
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
            $digitoAgencia = ' ';
        }
        
        //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
        $linha.= $digitoAgencia;

        //Nº da C/C Debitada - Posição: 59 a 70 - Picture: 9(12)
        //Obs: A última posição da C/C é reservada para o DAC
        $digitoContaCorrenteFlag   = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
        
        if($digitoContaCorrenteFlag){
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = substr(trim($this->contcorr->getCdcontcorr()), -1);
        }else{
            $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);
            $digitoContaCorrente = ' ';
        }

        //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
        $linha.= $digitoContaCorrente;

        //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
        $linha.= ' ';

        //Nome da empresa/filial debitada - Posição: 73 a 102 - Picture: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa, 0, 30)), 30,' ',STR_PAD_RIGHT);

        //Mensagem - Posição: 103 a 142 - Picture: X(40)
        $linha.= str_repeat(' ', 40);

        //Endereço da empresa - Posição: 143 a 172 - Picture: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($this->dsEndeEF, 0, 30)), 30,' ',STR_PAD_RIGHT);

        //Número do Local - Posição: 173 a 177 - Picture: 9(5)
        $linha.= str_repeat('0', 5);

        //Complemento (Casa, Apto, sala, etc) - Posição: 178 a 192 - Picture: X(15)
        $linha.= str_repeat(' ', 15);
        //Nome da cidade - Posição: 193 a 212 - Picture: X(20)
        $linha.= str_pad(Util::campoCnabValido(substr($this->muniEF, 0, 20)), 20,' ',STR_PAD_RIGHT);

        //CEP - Posição: 213 a 220 - Picture: 9(8)
        $linha.= str_pad(Util::campoCnabValido(substr($this->cepEF, 0, 8)), 8,' ',STR_PAD_RIGHT);

        //Sigla do Estado - Posição: 221 a 222 - Picture: X(2)
        $linha.= str_pad(Util::campoCnabValido($this->estadoEF), 2,' ',STR_PAD_RIGHT);

        //Brancos - Posição: 223 a 230 - Picture: X(8)
        $linha.= str_repeat(' ', 8);

        //Ocorrências p/ retorno: 231 a 240 - Picture: X(10)
        $linha.= str_repeat(' ', 10);

        return $linha."\r\n";
    }

    /**
     * @param Boletpagar $boleto
     * @param $cdbanco
     * @return string
     */
    private function geraSegmentoAJ(Boletpagar $boleto, $cdbanco)
    {

        $linha = '';
        $this->qtdRegiLote++;//Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha .= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha .= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(1)
        if (($this->formaPgto != '30') and ($this->formaPgto != '31'))
            $linha.= 'A';
        else
            $linha.= 'J';
        
        //Tipo de movimento - Posição: 15 a 15 - Picture: 9(1)
        if ($this->remessa->getIdopecnabpag() == 'E')
            $linha.= '9';
        else{
            $linha.= '0';
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }

        //Codigo de instrução para alteração - Posição:16 a 17 -Picture:9(2)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09';   // Bloqueio
        }

        //---------------------------------------------------------------------------------------------------------
        //Busca tipo de baixa
        /** @var $tipobaixapag Tipobaixapag */
        $tipobaixapag = Tipobaixapag::findOneByCdtpbaixapagAndNrorg($boleto->getCdtpbaixapag(), $this->nrorg );
        
        if (($this->formaPgto != '30') and ($this->formaPgto != '31')) {
            //Código da câmara centralizadora - Posição: 18 a 20 - Picture: 9(3)
            if ($this->formaPgto == '03' or $this->formaPgto == '45') {
                if(($tipobaixapag->getIdbaixautilp() == self::TIPO_BAIXA_PIX) || ($tipobaixapag->getIdbaixautilp() == self::TIPO_BAIXA_PIX_QRCODE)){
                    $linha.= '009'; //PIX
                }
                elseif (round($boleto->getVrfinalbpag(), 2) > 5000){
                    $linha .= '018';
                }else{
                    $linha .= '700';
                }
            } else {
                $linha .= '000';
            }
            
            //Código do Banco - Posição: 21 a 23 - Picture: 9(3)
            $linha .= str_pad(substr($cdbanco, 0, 3), 3, '0', STR_PAD_LEFT);

            if ($cdbanco == self::CODIGO_COMPENSACAO_BRASIL) {

                if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                    //Numero da agencia creditada - Posição: 24 a 28 - Picture: 9(5)
                    $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), 0, 4), 5, '0', STR_PAD_LEFT);

                    //Digito verificador da Agência- Posição: 29 a 29 - Picture: 9(1)
                    $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), -1), 1, ' ', STR_PAD_LEFT);
                } else {
                    //Numero da agencia creditada - Posição: 25 a 28 - Picture: 9(5)
                    $linha .= str_pad(substr(trim($boleto->getCdagenbolp()), 0, 4), 5, '0', STR_PAD_LEFT);

                    //Digito verificador da Agência- Posição: 29 a 29 - Picture: 9(1)
                    $linha .= str_pad(substr(trim($boleto->getCdagenbolp()), -1, 1), 1, ' ', STR_PAD_LEFT);
                }

                //Numero da Conta Corrente - Posição: 30 a 41 - Picture: 9(12)
                // CAMPO: Conta Favorecido (Sem DV)
                // POSIÇÃO: 30 a 41
                // PICTURE: 9(10)
                if (strpos($boleto->getCdccorbolp(), '-') === false) {
                    $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0, 12))), 12, '0', STR_PAD_LEFT);
                    //DV C/C - Posição: 42 a 42 - Picture: X(1)
                    $linha .= ' ';//str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(), -1))), 1, ' ', STR_PAD_LEFT);
                } else {
                    $linha .= substr(str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0,strpos($boleto->getCdccorbolp(), '-'))), 12, '0', STR_PAD_LEFT), -12);
                    //DV C/C - Posição: 42 a 42 - Picture: X(1)
                    $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(), (strpos($boleto->getCdccorbolp(), '-') + 1), 1))), 1, ' ', STR_PAD_LEFT);
                }
                
                //DV Agência e C/C - Posição: 43 a 43 - Picture: X(1)
                $linha .= ' ';
            } else {
                //Ag mantenedora da cta do favor - Posição: 24 a 28 - Picture: 9(5)
                if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                    $linha .= str_pad(substr($boleto->getReducont()->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);

                    //Digito verificador da Agência- Posição: 29 a 29 - Picture: 9(1)
                    $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), -1), 1, ' ', STR_PAD_LEFT);
                } else {
                    if (strpos($boleto->getCdagenbolp(), '-') === false) {
                        $linha .= str_pad(substr($boleto->getCdagenbolp(), 0, -1), 5, '0', STR_PAD_LEFT);
                        $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdagenbolp(), -1, 1))), 1, ' ', STR_PAD_LEFT);
                    } else {
                        $linha .= str_pad(substr($boleto->getCdagenbolp(), 0, strpos($boleto->getCdagenbolp(), '-')),
                            5, '0', STR_PAD_LEFT);
                        $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdagenbolp(),
                            (strpos($boleto->getCdagenbolp(), '-') + 1), 1))), 1, ' ', STR_PAD_LEFT);
                    }
                }
                
                //Numero da C/C creditada - Posição: 30 a 41 - Picture: 9(12)
                if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                    $linha .= str_repeat('0', 12);
                    $digitoConta = '0';
                } else {
                    if (strpos($boleto->getCdccorbolp(), '-') === false) {
                        $linha .= str_pad(trim(substr($boleto->getCdccorbolp(), 0, 12)), 12, '0', STR_PAD_LEFT);
                        $digitoConta = ' ';
                    } else {
                        $linha .= substr(str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0, strpos($boleto->getCdccorbolp(), '-'))), 12, '0', STR_PAD_LEFT), -12);
                        $digitoConta = trim(substr($boleto->getCdccorbolp(), (strpos($boleto->getCdccorbolp(), '-') + 1), 1));
                    }
                }
                
                //DV Conta Destino - Posição: 42 a 42 - Picture X(1)
                $linha .= $digitoConta;

                //DV Agência/ Conta Destino - Posição: 43 a 43 - Picture X(1)
                $linha .= ' ';
            }

            //Nome do favorecido - Posição: 44 a 73 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30,' ',STR_PAD_RIGHT);

            //Número do documento atribuído pela empresa - Posição: 74 a 93 - Picture: X(20)
            $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );

            //Data do pagamento - Posição: 94 a 101 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Tipo da Moeda - Posição: 102 a 104 - Picture: X(3)
            $linha.= self::MOEDA;

            //Quantidade da moeda - Posição: 105 a 119 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Valor do pagto - Posição: 120 a 134 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Nosso número - Posição: 135 a 149 - Picture: X(15)
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO)
                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,15)),15,' ',STR_PAD_RIGHT);
            else $linha.= str_repeat(' ', 15);

            //Brancos - Posição: 150 a 154 - Picture: X(5)
            $linha.= str_repeat(' ', 5);

            //Data efetiva - Posição: 155 a 162 - Picture: 9(8)
            $linha.= str_repeat('0', 8);

            //Valor efetiva - Posição: 163 a 177 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Finalidade Detalhe - Posição: 178 a 217 - Picture: X(40)
            $linha.=  str_repeat(' ', 40);

            //Brancos - Posição: 218 a 229 - Picture: X(12)
            $linha.=  str_repeat(' ', 12);

            //Aviso - Posição: 230 a 230 - Picture: X(1)
            $linha.= '0';
        }
        else{
            
            if ($boleto->getCdbarrapag()) {
                $cdbanco = str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
                $cdmoeda = str_pad(substr($boleto->getCdbarrapag(), 3, 1), 1, '0', STR_PAD_LEFT);
                $cddigito = str_pad(substr($boleto->getCdbarrapag(), 4, 1), 1, '0', STR_PAD_LEFT);
                $valor = str_pad(substr($boleto->getCdbarrapag(), 5, 14), 14, '0', STR_PAD_LEFT);
                $cdcampo = str_pad(substr($boleto->getCdbarrapag(), 19, 25), 25, '0', STR_PAD_LEFT);
            } else {

                $linhaBarra = '';
                $dslinhabarrapag = $boleto->getDslinhbarpag();
                for ($i = 0; $i < strlen($boleto->getDslinhbarpag()); $i++) {
                    if (in_array($dslinhabarrapag[$i], array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')))
                        $linhaBarra .= $dslinhabarrapag[$i];
                }

                $cdbanco = str_pad(substr($linhaBarra, 0, 3), 3, '0', STR_PAD_LEFT);
                $cdmoeda = str_pad(substr($linhaBarra, 3, 1), 1, '0', STR_PAD_LEFT);
                $cddigito = str_pad(substr($linhaBarra, 32, 1), 1, '0', STR_PAD_LEFT);
                $valor = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
                $cdcampo = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha .= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha .= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha .= $cddigito;

            //Valor do Documento (Cd. Barra) - Posição: 23 a 36 - Picture: 9(10)
            $linha .= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha .= $cdcampo;

            //Nome do favorecido - Posição: 62 a 91 - Picture: X(30)
            $linha .= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30, ' ', STR_PAD_RIGHT);

            //Data vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha .= $boleto->getDtvenbolpag()->format('dmY');

            //Valor previsto do pagto - Posição: 100 a 114 - Picture: 9(13)V9(2)
            if ($boleto->getCdbarrapag()){
              $linha.= str_pad(substr($boleto->getCdbarrapag(), 10, 9), 15,'0',STR_PAD_LEFT);
              $vrCodBarras = (int)(substr($boleto->getCdbarrapag(), 10, 9)) / 100;
              if ($boleto->getVrfinalbpag() - $vrCodBarras > 0) {
                  $vrdesconto = 0;
                  $vracrescimo = $boleto->getVrfinalbpag() - $vrCodBarras;
              }
              else {
                  $vrdesconto = $vrCodBarras - $boleto->getVrfinalbpag();
                  $vracrescimo = 0;
              }
           }
           else {
               $vrdesconto = 0;
               $vracrescimo = 0;
               $linha.= str_pad(str_replace(array('.',','),'',
                   number_format($boleto->getVrfinalbpag(),2)
               ),15,'0',STR_PAD_LEFT);
           }

            //Valor do Desconto - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($vrdesconto,2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($vracrescimo,2)
            ),15,'0',STR_PAD_LEFT);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Zeros - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Seu número - Posição: 183 a 202 - Picture: X(20)
            $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );

            //Nosso número - Posição: 203 a 222 - Picture: X(20)
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO)
                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,20)),20,' ',STR_PAD_RIGHT);
            else $linha.= str_repeat(' ', 20);

            //Brancos - Posição: 223 a 224 - Picture: 9(2)
            $linha.= '09';
        }

        $linha = str_pad($linha, 240);

        return $linha."\r\n";
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento B #########################//
     * //################################################################################//
     */
    private function geraSegmentoB(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'B';
        
        // Implementação de PIX segundo manual Febrabam
        if ($boleto->getIdchavepix()){
            //Identificação do favorecido - Posição: 15 a 17 - Picture: X(3)
            /* Para o tipo “forma de lançamento” igual ao PIX, usar o seguinte domínio.
            Domínio:
            o “01” – Chave Pix – tipo Telefone
            o “02” – Chave Pix – tipo Email
            o “03” – Chave Pix – tipo CPF/CNPJ
            o “04” – Chave Aleatória
            o “05” – Dados bancários
            */
            $linha.= str_pad($boleto->getIdchavepix(), 3,'0',STR_PAD_LEFT);

            // Tipo de Inscrição do Favorecido - Posição: 18 a 18 - Picture: 9(1)
            if($boleto->getIdtpicnabbop()) {
                if ($boleto->getIdtpicnabbop() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getIdtpicnabbop() == 'J')
                        $tipoInsc =  '2'; //CGC
                    else
                        $tipoInsc = '3'; //Outros
                }
            }
            else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getForneced()->getIdtpijurforn() == 'J')
                        $tipoInsc = '2'; //CGC
                    else
                     $tipoInsc = '3'; //Outros
                }
            }
            $linha.= $tipoInsc;
            
            //Nº de Inscrição do Favorecido - Posição: 19 a 32 - Picture: 9(14)
            if ($boleto->getNrinscnabbop())
               $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
            else
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
            
            // Dados Complementares - Informação 10 - Posição:  33 a 67 - Picture: X(35)
            /* Para o tipo “forma de lançamento” igual ao PIX, usar campo como:
                TX ID (opcional) Posição (33 67) Alfa.
            */
            $linha.= str_repeat(' ', 35);
            
            // Dados Complementares - Informação 11 - Posição:  68 a 127 - Picture: X(60)
            $linha.= str_repeat(' ', 60);
            
            // Dados Complementares - Informação 12 - Posição:  128 a 226 - Picture: X(99)
            $linha.= str_pad($boleto->getChavepix(), 99);
            
            // Uso Exclusivo para o SIAPE - Posição:   227 a 232 - Picture: 9(6)
            $linha.= str_repeat('0', 6);
            
            // Código ISPB - Posição: 233 a 240 - Picture: 9(8)
            $linha.= str_repeat(' ', 8);
        } else {
            //BRANCOS - Posição: 15 a 17 - Picture: 9(03)
            $linha.= str_repeat(' ', 3);
    
            //Tipo de incrição do fornecedor - Posição: 18 a 18 - Picture: 9(01)
            if($boleto->getIdtpicnabbop()) {
                if ($boleto->getIdtpicnabbop() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getIdtpicnabbop() == 'J')
                        $tipoInsc =  '2'; //CGC
                    else
                        $tipoInsc = '3'; //Outros
                }
            }
            else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'F')
                    $tipoInsc = '1'; //CPF
                else {
                    if ($boleto->getForneced()->getIdtpijurforn() == 'J')
                        $tipoInsc = '2'; //CGC
                    else
                     $tipoInsc = '3'; //Outros
                }
            }
            $linha.= $tipoInsc;
    
            //Código do CPF/CNPJ - Posição: 19 a 32 - Picture: 9(14)
            if ($boleto->getNrinscnabbop())
               $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
            else
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
    
            //Nome do logradouro do favorecido - Posição: 33 a 62 - Picture: X(30)
            $endforn = $boleto->getForneced()->getEndefornecobr();
            
            if(! $endforn)
                $endforn = $boleto->getForneced()->getEndeforneprinc();
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,30),30,' ',STR_PAD_RIGHT);
    
            //Número do imóvel do favorecido - Posição: 63 a 67 - Picture: 9(05)
            $linha.= str_repeat('0', 5);
    
            //Complemento do imóvel - Posição: 68 a 82 - Picture: X(15)
            $linha.= str_repeat(' ', 15);
    
            //Bairro do favorecido - Posição: 83 a 97 - Picture: 9(15)
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);
    
            //Município do favorecido - Posição: 98 a 117 - Picture: X(20)
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($endforn->getSgestado(), $endforn->getCdpais(), $endforn->getCdmunicipio());
            $linha.= str_pad(substr(Util::campoCnabValido($municipio->getNmmunicipio())
                ,0,20),20,' ',STR_PAD_RIGHT);
    
            //CEP - Posição: 118 a 122 - Picture: 9(05)
            if($endforn->getNrcepforn()){
                $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,5),5,'0');
            }else{
                $linha.= str_repeat('0',5);
            }
    
            //Complemento do CEP - Posição: 123 a 125 - Picture: X(03)
            if(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3)){
                $linha.= substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3);
    
            }else{
                $linha.= str_repeat(' ',3);
            }
    
            //Estado - Posição: 126 a 127 - Picture: X(02)
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);
    
            //Data do vencimento (Nominal) - Posição: 128 a 135 - Picture: 9(08)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
    
            //Valor do documento (Nominal) - Posição: 136 a 150 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);
    
            //Valor do abatimento - Posição: 151 a 165 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            //Valor do desconto - Posição: 166 a 180 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            //Valor da mora - Posição: 181 a 195 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            //Valor da multa - Posição: 196 a 210 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0', 15);
    
            //Codigo do Favorecido - Posição: 211 a 225 - Picture: X(15)
            $linha.= str_repeat(' ', 15);
    
            //Brancos - Posição: 226 a 240 - Picture: X(15)
            $linha.= str_repeat(' ', 15);
        }
        
        return $linha."\r\n";
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento J52 #######################//
     * //################################################################################//
     */
    private function geraSegmentoJ52(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'J';
        
        //Uso Exclusivo FEBRABAN/CNAB - Posição: 15 a 15 - Picture: X(01)
        $linha.= str_repeat(' ', 1);
        
        //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09'; // Bloqueio
        }
        
        //
        $linha.= '52';
        
        /*
         * #### PAGADOR ####
         */
        //Tipo de Inscrição Pagador - Posição: 20 a 20 - Picture: 9(1)
        $linha.= $this->tipoinscricao;
    
        //CPF/CNPJ do Pagador - Posição: 21 a 35 - Picture: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        //Nome do Pagador - Posição: 36 a 75 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        /*
         * #### Beneficiário ####
         */
        //Tipo de Inscrição Beneficiário - Posição: 76 a 76 - Picture: 9(01)
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
            $linha.= self::TIPO_INS_JUR_CPF;
        }else{
            $linha.= self::TIPO_INS_JUR_CNPJ;
        }
        
        //CPF/CNPJ do Beneficiário - Posição: 77 a 91 - Picture: 9(01)
        $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);

        //Nome do Beneficiário - Posição: 92 a 131 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
        
        /*
         * #### SACADOR ####
         */
        //Tipo de Inscrição Sacador - Posição: 132 a 132 - Picture: 9(1)
        //$linha.= $this->tipoinscricao;
        $linha.= str_repeat('0', 1);

        //CPF/CNPJ do Sacador - Posição: 133 a 147 - Picture: 9(15)
        //$linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        $linha.= str_repeat('0', 15);
        
        //Nome do Sacador - Posição: 148 a 187 - Picture: X(40)
        //$linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        $linha.= str_repeat(' ', 40);
        
        //Filler - Posição: 188 a 240 - Picture: X(53)
        $linha.= str_repeat(' ',53);

        return $linha . "\r\n";
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento O #########################//
     * //################################################################################//
     */
    private function geraSegmentoO(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;
        
        //Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
        $linha.= self::TIPO_REGISTRO_DETALHE;
        
        //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'O';
        
        //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
        if ($this->remessa->getIdopecnabpag() == 'E')
            $linha.= '9';
        else{
            $linha.= '0';
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }
        
        //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09';   // Bloqueio
        }
        //Código de Barras - Posição: 18 a 61 - Picture: X(44)
        if ($boleto->getCdbarrapag()) {
            $cdbanco = str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
            $cdmoeda = str_pad(substr($boleto->getCdbarrapag(), 3, 1), 1, '0', STR_PAD_LEFT);
            $cddigito = str_pad(substr($boleto->getCdbarrapag(), 4, 1), 1, '0', STR_PAD_LEFT);
            $valor = str_pad(substr($boleto->getCdbarrapag(), 5, 14), 14, '0', STR_PAD_LEFT);
            $cdcampo = str_pad(substr($boleto->getCdbarrapag(), 19, 25), 25, '0', STR_PAD_LEFT);
        } else {

            $linhaBarra = '';
            $dslinhabarrapag = $boleto->getDslinhbarpag();
            for ($i = 0; $i < strlen($boleto->getDslinhbarpag()); $i++) {
                if (in_array($dslinhabarrapag[$i], array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')))
                    $linhaBarra .= $dslinhabarrapag[$i];
            }

            $cdbanco = str_pad(substr($linhaBarra, 0, 3), 3, '0', STR_PAD_LEFT);
            $cdmoeda = str_pad(substr($linhaBarra, 3, 1), 1, '0', STR_PAD_LEFT);
            $cddigito = str_pad(substr($linhaBarra, 32, 1), 1, '0', STR_PAD_LEFT);
            $valor = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
            $cdcampo = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
        }
        //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
        $linha .= $cdbanco;

        //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
        $linha .= $cdmoeda;

        //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
        $linha .= $cddigito;

        //Valor do Documento (Cd. Barra) - Posição: 23 a 36 - Picture: 9(10)
        $linha .= $valor;

        //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
        $linha .= $cdcampo;

        //Nome Concessionária - Posição: 62 a 91 - Picture: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
        
        //Data de Vencimento - Posição: 92 a 99 - Picture: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Data do Pagamento - Posição: 100 a 107 - Picture: X(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Valor do Pagamento - Posição: 108 a 122 - Picture: 9(15)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Número Documento do Cliente - Posição: 123 a 142 - Picture: 9(20)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );
        
        //Número Documento atribuido pelo Banco - Posição: 143 a 162 - Picture: X(20)
        $linha.= str_repeat(' ', 20);
        
        //Posição: Filler 163 a 230 - Picture: X(68)
        $linha.= str_repeat(' ', 68);
        
        //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
        $linha.= str_repeat(' ', 10);
        
        return $linha . "\r\n";
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento N #########################//
     * //################################################################################//
     */
    private function geraSegmentoN(Boletpagar $boleto){

        $linha = '';
                
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        
        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;
        
        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_DETALHE;
        
        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(1)
        $linha.= 'N';
        
        //Tipo de Movimento - Posição: 15 a 15 - Picture: 9(1)
        if ($this->remessa->getIdopecnabpag() == 'E')
            $linha.= '9';
        else{
            $linha.= '0';
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }
        
        //Codigo de instrução para alteração - Posição:16 a 17 -Picture:9(2)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09';   // Bloqueio
        }
        
        //N do doc.Atribuido p/empresa - Posição: 18 a 37 - Picture: X(20)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );
        
        //Nosso número - Posição: 38 a 57 - Picture: X(20)
        if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,20)),20,' ',STR_PAD_RIGHT);
        } else {
            $linha.= str_repeat(' ', 20);
        }
        
        //Nome do favorecido - Posição: 58 a 87 - Picture: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30,' ',STR_PAD_RIGHT);
        
        //Data do pagamento - Posição: 88 a 95 - Picture: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Valor do pagto - Posição: 96 a 110 - Picture: 9(13)V9(2)
        $linha .= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,STR_PAD_LEFT);
        
        //Informações Complementares de acordo com o tributo
        //Posição: 111 a 230
         if ($this->formaPgto == '16')
                $linha = self::darfNormal($linha, $boleto);
            elseif($this->formaPgto == '17')
                $linha = self::GPS($linha, $boleto);
            elseif ($this->formaPgto == '18')
                $linha = self::darfSimples($linha, $boleto);
            elseif ($this->formaPgto == '21')
                $linha = self::DARJ($linha, $boleto);
            elseif ($this->formaPgto == '22')
                $linha = self::GARE($linha, $boleto);
                
        //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)  
        $linha.= str_repeat(' ', 10);

        return $linha . "\r\n";
    }
    
     /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro Tributo FGTS       #########################//
     * //################################################################################//
     */
    private function FGTS($linha, $boleto){
        
        //IDENTIFICAÇÃO DO TRIBUTO - Posição: 177 a 178 - Picture: 9(02)
        $linha.= '01'; //FGTS
        
        //CÓDIGO DA RECEITA - Posição: 179 a 184 - Picture: 9(06)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        //TIPO DE INSCRIÇÃO DO CONTRIBUINTE - Posição: 185 a 186 - Picture: 9(02)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '2 '; //CPF
        else
            $linha.= '1 '; //CNPJ
        
        //CPF ou CNPJ do contribuinte  - Posição: 187 a 200 - Picture: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        //IDENTIFICADOR DO FGTS - Posição: 201 - 216 - Picture: 9(16)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!$nrrefeguia){
            $linha.= str_pad($nrrefeguia,16,'0',STR_PAD_LEFT);
        } else {
            $linha.= str_repeat('0', 16);
        }
        //LACRE DE CONECTIVIDADE SOCIAL - Posição: 217 - 225 - Pictures: 9(09)
        // if($boleto->getNrlacrconectsoc()){
        //     $linha.= str_pad(substr($boleto->getNrlacrconectsoc(),0,9),9);
        // } else {
        //     $linha.= str_repeat('0', 9);
        // }
        
        $linha.= str_repeat('0', 9);
        
        //DIGITO DO LACRE DE CONECTIVIDADE SOC. - Posição: 226 - 227 - Picture: 9(02)
        // if($boleto->getNrdigiconectsoc()){
        //     $linha.=$linha.= str_pad(substr($boleto->getNrdigiconectsoc(),0,2),2);
        // } else {
        //      $linha.= str_repeat('0', 2);
        // }
        $linha.= str_repeat('0', 2);
        
        //BRANCOS - Posição: 228 a 228 - Picture: X(1)
        $linha.= str_repeat(' ',1);
        
        return $linha;
    
    }
    
    
     /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento W #########################//
     * //################################################################################//
     */
    private function geraSegmentoW(Boletpagar $boleto){
        
        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;
        
        //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;
        
        //Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);
        
        //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
        $linha.= self::TIPO_REGISTRO_DETALHE;
        
        //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);
        
        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'W';
        
        //NÚMERO SEQUENCIAL DE REGISTRO COMPLEMENTAR - Posição: 15 a 15 - Picture: 9(01)
        $linha.= str_repeat('0', 1);
        
        //IDENTIFICA O USO DAS INFORMAÇÕES 1 E 2 - Posição: 16 a 16 - Picture: 9(01)
        $linha.= str_repeat('1', 1);
        
        //INFORMAÇÃO 1 - Posição: 17 a 96 - Picture: X(80)
        $linha.= str_repeat(' ', 80);
        
        //INFORMAÇÃO 2 - Posição: 97 a 176 - Picture: X(80)
        $linha.= str_repeat(' ', 80);
        
        //INFORMAÇÃO COMPLEMENTAR DO TRIBUTO FGTS - Posição: 177 a 228 - FGTS;
        $linha = self::FGTS($linha, $boleto);
        
        //FILLER - Posição: 229 a 230 - Picture: X(02)
        $linha.= str_repeat(' ', 2);
        
        //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
        $linha.= str_repeat(' ', 10);
        
        return $linha . "\r\n";
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro Tributo DarfSimples #########################//
     * //################################################################################//
     */
    private function darfSimples($linha, $boleto){
        
        //Código da receita - Posição: 111 a 116 - Picture: 9(06)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '02'; //CPF
        else
            $linha.= '01'; //CNPJ
        
        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        //Identificação do tributo - Posição: 133 a 134 - Picture: 9(02)
        $linha.= $this->formaPgto;
        
        //Período de Apuração - Posição: 135 a 142 - Picture: 9(08)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmY');
        } else {
            $linha.= str_repeat('0', 8);
        }
        
        //Valor da Receita Bruta Acumulada - Posição: 143 a 157 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
         
        //Percentual - Posição: 158 a 164 - Picture: 9(5)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrpercenttrib(),2)),7,'0',STR_PAD_LEFT);
        
        //Valor Principal - Posição: 165 a 179 - Picture: 9(13)V2(02)
        if($boleto->getVrpercenttrib() != 0 && $boleto->getVroutrentguia() != 0 ){
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia() * $boleto->getVrpercenttrib(),2)),15,STR_PAD_LEFT);
        } else {
             $linha .= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,STR_PAD_LEFT);
        }
        
        //Valor da Multa - Posição: 180 a 194 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor dos Juros/ Encargos - Posição: 195 a 209 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Brancos - Posição: 210 a 230 - Picture: X(21)
        $linha = str_pad($linha, 230);
        
        return $linha;
    
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro Tributo DarfNormal #########################//
     * //################################################################################//
     */
    private function darfNormal($linha, $boleto){
        
        //Código da receita - Posição: 111 a 116 - Picture: 9(06)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
         
        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '02'; //CPF
        else
            $linha.= '01'; //CNPJ
        
        //CPF ou CNPJ do contribuinte - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        //Identificação do tributo - Posição: 133 a 134 - Picture: 9(02)
        $linha.= $this->formaPgto;
        
        //Período de Apuração - Posição: 135 a 142 - Picture: 9(08)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmY');
        } else {
            $linha.= str_repeat('0', 8);
        }
        
        //Número de referência - Posição: 143 a 159 - Picture: 9(17)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!$nrrefeguia){
            $linha.= str_pad($nrrefeguia,17,'0',STR_PAD_LEFT);
        } else {
            $linha.= str_repeat('0', 17);
        }
        
        //Valor Principal - Posição: 160 a 174 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 175 a 189 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor dos Juros/ Encargos - Posição: 190 a 204 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 205 a 212 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Brancos - Posição: 213 a 230 - Picture: X(18)
        $linha = str_pad($linha, 230);
        
        return $linha;
    
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro Tributo GPS        #########################//
     * //################################################################################//
     */
     private function GPS($linha, $boleto){
         
        //Código de pagamento - Posição: 111 a 116 -Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '02'; //CPF
        else
            $linha.= '01'; //CNPJ
        
        //CPF ou CNPJ do contribuinte - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        //Identificação do tributo - Posição: 133 a 134 - Picture: 9(02)
        $linha.= $this->formaPgto;
       
        //Mês e ano da Competência - Posição: 135 a 140 - Picture: 9(06)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmy');
        } else {
            $linha.= str_repeat('0', 6);
        }
        
        //Valor previsto do pagamento do INSS - Posição: 141 a 155 - Picture: 9(13) V9(02)
        $vrprevisto = $boleto->getVrfinalbpag() - ($boleto->getVroutrentguia() + $boleto->getVrmultabpag() + $boleto->getVrmordiabpag());
        $linha.= str_pad(str_replace(array('.',','),'', number_format($vrprevisto,2)),15,'0',STR_PAD_LEFT);
        
        //Valor de outras entidades - Posição: 156 a 170 - Picture: 9(13) V9(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
        
        //Atualização Monetária - Posição: 171 a 185 - Picture: 9(13) V9(02)
        $vratualizacao = $boleto->getVrmultabpag() + $boleto->getVrmordiabpag();
        $linha.= str_pad(str_replace(array('.',','),'', number_format($vratualizacao,2)),15,'0',STR_PAD_LEFT);

        //COMPLEMENTO DO REGISTRO - Posição: 186 a 230 - Picture: X(45)
        $linha = str_pad($linha, 230);
        
        return $linha;
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro Tributo GARE        ########################//
     * //################################################################################//
     */
     private function GARE($linha, $boleto){
         
        //Código da receita - Posição: 111 a 116 - Picture: 9(06)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
       if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '02'; //CPF
        else
            $linha.= '01'; //CNPJ
        
        //CPF ou CNPJ do contribuinte  - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
        
        //Identificação do tributo - Posição: 133 a 134 - Picture: 9(02)
        $linha.= $this->formaPgto;
        
        //Data de Vencimento - Posição: 135 a 142 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Inscrição Estadual - Posição: 143 a 154 - Picture: 9(12)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        $linha.= str_pad($empresa->getCdinscesta(),12, '0', STR_PAD_LEFT);
        
        //Divida Ativa / Número Etiqueta - Posição: 155 a 167 - Picture: 9(13)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!$nrrefeguia){
            $linha.= str_pad($nrrefeguia,13,'0',STR_PAD_LEFT);
        } else {
            $linha.= str_repeat('0', 13);
        }
        
        //Mês e Ano de Referência - Posição: 168 a 173 - Picture: 9(06)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmy');
        } else {
            $linha.= str_repeat('0', 6);
        }

        //Número Parcela/Notificação - Posição: 174 a 186 - Picture: 9(13)
        // $parcelaNotific = $boleto->getNrparcelnotific();
        // if($parcelaNotific){
        //     $linha.= $parcelaNotific;
        // }else {
        //     $linha.= str_repeat('0', 13);
        // }
        
        $linha.= str_repeat('0', 13);
        
        //Valor da Receita - Posição: 187 a 201 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor dos Juros/Encargos - Posição: 202 a 215 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 216 a 229 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Brancos - Posição: 230 a 230 - Picture: X(1)
        $linha.= str_repeat(' ', 1);

        return $linha;
    }
    
     /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro Tributo DARJ        ########################//
     * //################################################################################//
     */
     private function DARJ($linha, $boleto){

        //Código da receita - Posição: 111 a 116 - Picture: 9(06)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if ($cdreceitaguia){
             $linha.= str_pad($cdreceitaguia,6,'0',STR_PAD_LEFT);
        } else { 
            $linha.= str_repeat('0', 6);
        }
        
        //Tipo inscrição empresa debitada - Posição: 117 a 118 - Picture: 9(2)
        if ($this->empresa->getIdtpijurempr() == 'F')
            $linha.= '01'; //CPF
        else
            $linha.= '02'; //CNPJ
        
        //CPF ou CNPJ do contribuinte - Posição: 119 a 132 - Picture: 9(14)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),14,'0', STR_PAD_LEFT);
         
        //Inscrição Estadual - Posição: 133 a 140 - Picture: 9(08)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        $linha.= str_pad($empresa->getCdinscesta(),8, '0', STR_PAD_LEFT);
        
        //Número Documento Origem - Posição: 141 a 156 - Picture: 9(16)
        $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),$boleto->getCdfilial(),$boleto->getNrorg());
        $nrnotafisc = $contaPagar->getLanctonf()->getNrnotafisc();
        if($nrnotafisc){
            $linha.= str_pad($nrnotafisc,16,'0',STR_PAD_RIGHT);
        } else{
            $linha.= str_repeat('0', 16);
        }
        
        //Valor Principal - Posição: 157 a 171 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag() - $boleto->getVrmultabpag() + $boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor Atualização Monetária - Posição: 172 a 186 - Picture: 9(13)V2(02)
        $linha.= str_repeat('0', 15);
        
        //Valor dos Juros/ Encargos - Posição: 187 a 201 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 202 a 216 - Picture: 9(13)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),15,'0',STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 217 a 224 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Período de Referência - Posição: 225 a 230 - Picture: 9(06)
        if ($boleto->getDtapuracguia()){
            $linha.= $boleto->getDtapuracguia()->format('dmy');
        } else {
            $linha.= str_repeat('0', 6);
        }
        
        return $linha;
    }
    
    /**
     *#################################################################//
    //##################  Registro Trailler de Lote ###################//
    //#################################################################//
     */
     
    public function geraTraillerLote($somavalores = null)
    {

        $this->qtdRegiLote++; //Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha = self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote do serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha.= str_pad(($this->num_lote), 4,'0',STR_PAD_LEFT);

        //Tipo de registro = Registro Trailer do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_TRAILLER_LOTE;

        //Brancos - Posição: 9 a 17 - Picture: X(9)
        $linha.= str_repeat(' ', 9);

        //Total qtd Registro do lote - Posição: 18 a 23 - Picture: 9(6)
        $linha.= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

        //Valor do pagto - Posição: 24 a 41 - Picture: 9(16)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'',
            number_format($this->vrPagtLote,2)
        ),18,'0',STR_PAD_LEFT);

        //Zeros - Posição: 42 a 59 - Picture: 9(18)
        $linha.= str_repeat('0', 18);

        //Brancos - Posição: 60 a 65 - Picture: X(6)
        $linha.= str_repeat('0', 6);

        //Brancos - Posição: 66 a 230 - Picture: X(165)
        $linha.= str_repeat(' ', 165);

        //Ocorrências - Posição: 231 a 240 - Picture: X(10)
        $linha.= str_repeat(' ', 10);

        return $linha."\r\n";

    }

    public function geraTrailerArquivo()
    {
        //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
        $linha = self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote do serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha.= self::LOTE_DE_SERVICO_TRAILLER_ARQUIVO;

        //Tipo de registro = registro trailer de arquivo - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_TRAILLER_ARQUIVO;

        //Brancos - Posição: 9 a 17 - Picture: X(9)
        $linha.= str_repeat(' ', 9);

        //Total Qtde de lotes do arquivo - Posição: 18 a 23 - Picture: 9(6)
        $linha.= str_pad($this->num_lote, 6,'0',STR_PAD_LEFT);

        //Total de Qtde de registro do arquivo - Posição: 24 a 29 - Picture: 9(6)
        $linha.= str_pad(++$this->qtdRegiArq, 6,'0',STR_PAD_LEFT);

        //Brancos - Posição: 30 a 35 - Picture: 9(6)
        $linha.= str_repeat('0', 6);

        //Brancos - Posição: 36 a 240 - Picture: X(205)
        $linha.= str_repeat(' ', 205);

        return $linha."\r\n";
    }
    
    /**
     * @param Boletpagar $boleto
     * @param $cdbanco
     * @return string
     //################################################################################//
     * //##############################################################################//
     * //#########################  Registro detalhe segmento J PIX ###################//
     * //##############################################################################//
     */
    private function geraSegmentoJPix(Boletpagar $boleto, $cdbanco)
    {
        $linha = '';
        $this->qtdRegiLote++;//Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha .= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha .= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(1)
        $linha.= 'J';
        
        //Tipo de movimento - Posição: 15 a 15 - Picture: 9(1)
        if ($this->remessa->getIdopecnabpag() == 'E')
            $linha.= '9';
        else{
            $linha.= '0';
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }

        //Codigo de instrução para alteração - Posição:16 a 17 -Picture:9(2)
        if ($this->remessa->getIdopecnabpag() == 'E'){
           $linha .= '99'; //Exclusão
        }else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09';   // Bloqueio
        }
        
        $cdbanco   = '000';                 //PARA PIX PREENCHER COM ZEROS
        $cdmoeda   = '0';                   //PARA PIX PREENCHER COM ZEROS
        $cddigito  = '0';                   //PARA PIX PREENCHER COM ZEROS
        $fatorvenc = '0000';                //PARA PIX PREENCHER COM ZEROS
        $valor     = str_repeat('0',10);    //PARA PIX PREENCHER COM ZEROS
        $cdcampo   = str_repeat('0',25);    //PARA PIX PREENCHER COM ZEROS
        
        //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
        $linha .= $cdbanco;

        //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
        $linha .= $cdmoeda;

        //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
        $linha .= $cddigito;

        //Valor do Documento (Cd. Barra) - Posição: 23 a 36 - Picture: 9(10)
        $linha .= $valor;

        //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
        $linha .= $cdcampo;

        //Nome do favorecido - Posição: 62 a 91 - Picture: X(30)
        $linha .= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30, ' ', STR_PAD_RIGHT);

        //Data vencimento - Posição: 92 a 99 - Picture: 9(8)
        $linha .= $boleto->getDtvenbolpag()->format('dmY');

        //Valor previsto do pagto - Posição: 100 a 114 - Picture: 9(13)V9(2)

        $vrdesconto = 0;
        $vracrescimo = 0;
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

        //Valor do Desconto - Posição: 115 a 129 - Picture: 9(13)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($vrdesconto,2)),15,'0',STR_PAD_LEFT);
        
        //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($vracrescimo,2)),15,'0',STR_PAD_LEFT);

        //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');

        //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'',number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

        //Zeros - Posição:  168 a 182 - Picture: 9(15)
        $linha.= str_repeat('0', 15);

        //Seu número - Posição: 183 a 202 - Picture: X(20)
        $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );

        //Nosso número - Posição: 203 a 222 - Picture: X(20)
        if($this->remessa->getIdopecnabpag() == self::EXCLUSAO)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,20)),20,' ',STR_PAD_RIGHT);
        else $linha.= str_repeat(' ', 20);

        //Brancos - Posição: 223 a 224 - Picture: 9(2)
        $linha.= '09';

        $linha = str_pad($linha, 240);
        
        return $linha."\r\n";
    }
    
        /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#####################  Registro detalhe segmento J52 Pix #######################//
     * //################################################################################//
     */
    private function geraSegmentoj52Pix(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'J';
        
        //Uso Exclusivo FEBRABAN/CNAB - Posição: 15 a 15 - Picture: X(01)
        $linha.= str_repeat(' ', 1);
        
        //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09'; // Bloqueio
        }
        
        //
        $linha.= '52';
        
        /*
         * #### PAGADOR ####
         */
        //Tipo de Inscrição Pagador - Posição: 20 a 20 - Picture: 9(1)
        $linha.= $this->tipoinscricao;
    
        //CPF/CNPJ do Pagador - Posição: 21 a 35 - Picture: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        //Nome do Pagador - Posição: 36 a 75 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        /*
         * #### Beneficiário ####
         */
        //Tipo de Inscrição Beneficiário - Posição: 76 a 76 - Picture: 9(01)
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
            $linha.= self::TIPO_INS_JUR_CPF;
        }else{
            $linha.= self::TIPO_INS_JUR_CNPJ;
        }
        
        //CPF/CNPJ do Beneficiário - Posição: 77 a 91 - Picture: 9(01)
        $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);

        //Nome do Beneficiário - Posição: 92 a 131 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
        
        //Chave de Pagamento  - Posição: 132 a 210 - Picture: X(79)
        $linha.= str_pad(' ',79);
        
        //TXID - Opcional - Posição: 211 a 240 - Picture: X(30)
        $linha.= str_repeat(' ',30);

        return $linha . "\r\n";
    }
    
    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null)
    {
        try{
            $file = fopen($caminhoArquivo.'.REM', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);
            
            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);
            
            $headerLoteGerado = false;
            
            //Percorre as formas de pagamento
            /** @var $boleto Boletpagar  */
            foreach($boletosAgrup as $segmento => $boletosFormaPgo){
                //Gera Segmentos AB
                if($segmento == 'SEGMENTOAB'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);
                        foreach($boletosBco as $boleto){
                            $segmentoA = self::geraSegmentoAJ($boleto, $boleto->getCdbancbolp());
                            $segmentoB = self::geraSegmentoB($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoA);
                            fprintf($file, "%s",$segmentoB);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTOJ'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);
                        foreach($boletosBco as $boleto){
                            $segmentoJ = self::geraSegmentoAJ($boleto, $boleto->getCdbancbolp());
                            $segmentoj52 = self::geraSegmentoj52($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoJ);
                            fprintf($file, "%s",$segmentoj52);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTOO'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);
                        foreach($boletosBco as $boleto){
                            $segmentoO = self::geraSegmentoO($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoO);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                if($segmento == 'SEGMENTON'){
                    //Percorre os bancos
                    foreach($boletosFormaPgo as $key => $boletosBco){
                        $headerlote  = self::geraHeaderLote($key, $boletosBco);
                        fprintf($file, "%s",$headerlote);
                        foreach($boletosBco as $boleto){
                            $segmentoN = self::geraSegmentoN($boleto, $boleto->getCdbancbolp());
                            fprintf($file, "%s",$segmentoN);
                        }
                        $trailerLote = self::geraTraillerLote();
                        fprintf($file, "%s",$trailerLote);
                    }
                }
                // if($segmento == 'SEGMENTOJPIX'){
                //     //Percorre os bancos
                //     foreach($boletosFormaPgo as $key => $boletosBco){
                //         $headerlote  = self::geraHeaderLote($key, $boletosBco);
                //         fprintf($file, "%s",$headerlote);
                //         foreach($boletosBco as $boleto){
                //             $segmentoJ = self::geraSegmentoJPix($boleto, $boleto->getCdbancbolp());
                //             $segmentoj52 = self::geraSegmentoj52Pix($boleto, $boleto->getCdbancbolp());
                //             fprintf($file, "%s",$segmentoJ);
                //             fprintf($file, "%s",$segmentoj52);
                //         }
                //         $trailerLote = self::geraTraillerLote();
                //         fprintf($file, "%s",$trailerLote);
                //     }
                // }
                
            }
            fprintf($file, "%s",self::geraTrailerArquivo());
            fclose($file);
            
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }
    
    /** 
     * Retorna array de Boletos Agrupados por 
     * Forma de Pagamento e por código do banco favorecido
     */
    private function agrupaBoletos($boletos){
        $boletosAgroup = array();

        $segmentosAB = array(
            Tipobaixapag::TIPOBAIXA_CNAB,
            Tipobaixapag::TIPOBAIXA_CNAB_DOC,
            Tipobaixapag::TIPOBAIXA_TED_CIP,
            Tipobaixapag::TIPO_BAIXA_TED_STR,
            self::TIPO_BAIXA_PIX,
        );

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);
        
        //PIX QRCODE
        // $segmentoJPIX = array('Z');

        $segmentoO = array(
            Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS, 
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL, 
            Tipobaixapag::TIPO_BAIXA_FGTS,
            self::TIPO_BAIXA_PAG_TRIBUTO_MUNICIPAL,
            self::TIPO_BAIXA_PAG_DAE
        );
        
        $segmentoN = array(
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES,
            Tipobaixapag::TIPO_BAIXA_GPS,
            self::TIPO_BAIXA_PAG_IPVA,
            self::TIPO_BAIXA_PAG_GARE,
            self::TIPO_BAIXA_PAG_DARJ
        );

        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            
            /** @var Boletpagar $boleto */
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag(
                $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']
            );

            /** @var Tipobaixapag $tipobaixapag */
            $tipobaixapag = Tipobaixapag::findOneByNrorgAndCdtpbaixapag($boleto->getNrorg(), $boleto->getCdtpbaixapag());
            
            $idbaixautilp = $tipobaixapag?$tipobaixapag->getIdbaixautilp():null;
            
            if(in_array($idbaixautilp,$segmentosAB)){

                if(self::getBanco($boleto) == GeraCnabService::CODIGO_COMPENSACAO_BRASIL)
                     $boletosAgroup['SEGMENTOAB'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                else $boletosAgroup['SEGMENTOAB'] ['OUTROS_BANCOS'] [] = $boleto;

            }else{
                if(in_array($idbaixautilp,$segmentoJ)){
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                         $boletosAgroup['SEGMENTOJ'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                    else $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                }else{
                    if(in_array($idbaixautilp,$segmentoO)){
                        $cdbanco = self::getBanco($boleto, $idbaixautilp);

                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                             $boletosAgroup['SEGMENTOO'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                        else $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;

                    }else{
                        if(in_array($idbaixautilp,$segmentoN)){
                            $cdbanco = self::getBanco($boleto, $idbaixautilp);
    
                            if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                                 $boletosAgroup['SEGMENTON'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                            else $boletosAgroup['SEGMENTON'] ['OUTROS_BANCOS'] [] = $boleto;
                            
                        // }elseif(in_array($idbaixautilp,$segmentoJPIX)){
                        //     $cdbanco = self::getBanco($boleto, $idbaixautilp);
                        //     if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_BRASIL))
                        //          $boletosAgroup['SEGMENTOJPIX'] [GeraCnabService::CODIGO_COMPENSACAO_BRASIL] [] = $boleto;
                        //     else $boletosAgroup['SEGMENTOJPIX'] ['OUTROS_BANCOS'] [] = $boleto;
                        } else throw new \Exception ('Operação bloqueada. Tipo de Baixa não reconhecido.');
                        
                    }
                }
            }
        }
        
        return $boletosAgroup;
    }

    /**
     * @param $cdbanco
     * @param Boletpagar $boleto
     */
    private function getFormaPagto($cdbanco, $boletos){
        
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());

            $idbaixa = $boleto->getTpbaixapag()->getIdbaixautilp();
        }

        if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG)
           return '10';
        else if ($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS || $idbaixa == self::TIPO_BAIXA_PAG_DAE)
            return '11';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_GPS)
            return '17';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES)
            return '18';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL)
            return '16';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_DARJ)
            return '21';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_FGTS)
            return '99';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_GARE)
            return '22';
        else if($idbaixa == self::TIPO_BAIXA_PIX)
            return '45';
        // else if($idbaixa == self::TIPO_BAIXA_PIX_QRCODE)
        //     return '47';
        else{
            if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($cdbanco == self::CODIGO_COMPENSACAO_BRASIL)
                    return '30'; //Liquidação de Títulos do Próprio Banco
                else
                    return '31'; //Pagamento de Títulos de Outros Bancos
            }else{
                if($cdbanco == self::CODIGO_COMPENSACAO_BRASIL)
                    return '01'; //Crédito em conta corrente
                else{
                    return '03'; //DOC/TED
                }
            }
        }

    }
    
    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }
}