<?php

namespace Teknisa\FIN\Modules\ContaReceber\Controllers;

use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Rec15000ChequeRecebidoController extends TeknisaCrud{
    protected $connection;
    protected $service;
    
    public function __construct($crudService, $parameterBag, $service, $entityManager) {
        $this->service = $service;
        $this->connection = $entityManager->getConnection();
        parent::__construct($crudService, $parameterBag);
    }
    
    public function setCustodia(DTO\Request $request, DTO\Response $response) {
        try {
            $tipo = $request->getDataSet()->getRows()[0]['tipo'];
            $rows = $request->getDataSet()->getRows()[0]['dados'];
            $dataSet = $this->service->setCustodia($tipo, $rows);
            $response->addDataSet($dataSet);
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function gerarApresentacao(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try {
            $apresentacao = $request->getDataSet()->getRows()[0]['apresentacao'];
            $rows = $request->getDataSet()->getRows()[0]['dados'];
            $dataSet = $this->service->gerarApresentacao($apresentacao, $rows);
            $response->addDataSet($dataSet);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
    
    public function gerarDevolucao(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try{
            $apresentacao = $request->getDataSet()->getRows()[0]['apresentacao'];
            $rows = $request->getDataSet()->getRows()[0]['dados'];
            foreach($rows as $row){
                $dataSet = $this->service->gerarDevolucao($apresentacao, $row);
                $response->addDataSet($dataSet);
            }
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
    
    public function gerarCompensacao(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try{
            $compensacao = $request->getDataSet()->getRows()[0]['compensacao'];
            $row = $request->getDataSet()->getRows()[0]['cheques'];
            if($compensacao['IDSTATUSBREC'] === 'P') $dataSet = $this->service->setBaixa($compensacao, $row, 'P', $response);
            else if($compensacao['IDSTATUSBREC'] === 'R') $dataSet = $this->service->setBaixa($compensacao, $row, 'E', $response);
            else throw new \Exception('Baixa invalida.');
            $response->addDataSet($dataSet);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
    
    public function getBoletosCompensacao(DTO\Request $request, DTO\Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions(); 
            $dataSet = new DataSet('getBoletosCompensacao', $this->service->getDadosCompensacao($filter, 1));
            $response->addDataSet($dataSet);
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getChequesCompensacao(DTO\Request $request, DTO\Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions(); 
            $dataSet = new DataSet('getChequesCompensacao', $this->service->getDadosCompensacao($filter, 2));
            $response->addDataSet($dataSet);
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deletarCheque(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try {
            $row = $request->getRow();
            $conditions = $this->service->getConditionsDelete($row);
            $rowsDeleteAp = $this->crudService->getFilteredDataSet('logchequerec', $conditions);
            $rowsDeleteCh = $this->crudService->getFilteredDataSet('chequerece', $conditions);
            if(isset($rowsDeleteAp->getRows()[0])) {
                $this->service->deletarApresentacoes($rowsDeleteAp);
            }
            $this->crudService->delete($response, $rowsDeleteCh);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
    
    public function deleteBoleto(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try {
            $row = $request->getRow();
            $this->service->deleteBoleto($row);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
    
    public function saveBoleto(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try {
            $dataSet = $request->getDataSet();
            $rowsToSave = $dataSet->getRows();
            foreach ($rowsToSave as $row) {
                $this->service->saveBoleto($row);
                //$this->connection->flush();
            }
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
} 