<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util;

class ApiItau{

  /**
   * URL base do PSP
   * @var string
   */
  private $baseUrl;

  /**
   * Client ID do oAuth2 do PSP
   * @var string
   */
  private $clientId;

  /**
   * Client secret do oAuht2 do PSP
   * @var string
   */
  private $clientSecret;

  /**
   * Caminho absoluto até o arquivo do certificado
   * @var string
   */
  private $certificate;

  /**
   * Define os dados iniciais da classe
   * @param string $baseUrl
   * @param string $clientId
   * @param string $clientSecret
   * @param string $certificate
   */
  public function __construct($clientId,$clientSecret,$certificate,$homologacao = true){
    if($homologacao)
         $baseUrl = 'https://sandbox.devportal.itau.com.br';
    else $baseUrl = 'https://secure.api.itau';
    
    $this->baseUrl      = $baseUrl;
    $this->clientId     = $clientId;
    $this->clientSecret = $clientSecret;
    $this->certificate  = $certificate;
  }

  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function createCob($txid,$request){
    return $this->send('PUT','/v2/cob/'.$txid,$request);
  }


  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function createCobV($txid,$request){
    return $this->send('PUT','/itau-ep9-gtw-pix-recebimentos-ext-v2/v2/cobv/'.$txid, $request);
  }


  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function updateCobV($txid,$request){
    return $this->send('PATCH','/itau-ep9-gtw-pix-recebimentos-ext-v2/v2/cobv/'.$txid, $request);
  }

  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function getCobV($txid,$request){
    return $this->send('GET','/itau-ep9-gtw-pix-recebimentos-ext-v2/v2/cobv/'.$txid, $request);
  }

  /**
   * Método responsável por consultar uma cobrança imediata
   * @param  string $txid
   * @return array
   */
  public function consultCob($txid){
    return $this->send('GET','/pix');
  }


  /**
   * Método responsável por obter o token de acesso às APIs Pix
   * @return string
   */
  private function getAccessToken(){
    //ENDPOINT COMPLETO
    $endpoint = $this->baseUrl.'/api/oauth/jwt';
    
    //HEADERS
    $headers = [
        'Content-Type: application/x-www-form-urlencoded',
        'User-Agent: insomnia/2023.5.5'
    ];

    //CORPO DA REQUISIÇÃO
    $request = 'grant_type=client_credentials&client_id='.$this->clientId.'&client_secret='.$this->clientSecret;
    
    //CONFIGURAÇÃO DO CURL
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => $endpoint,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => '',
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => 'POST',
      CURLOPT_POSTFIELDS => $request,
      CURLOPT_HTTPHEADER => $headers
    ));
    
    //EXECUTA O CURL
    $response = curl_exec($curl);
    curl_close($curl);

    //RESPONSE EM ARRAY
    $responseArray = json_decode($response,true);

    //RETORNA O ACCESS TOKEN
    return $responseArray['access_token'] ?? '';
  }


  /**
   * Método responsável por enviar requisições para o PSP
   * @param  string $method
   * @param  string $resource
   * @param  array  $request
   * @return array
   */
  private function send($method,$resource,$request = []){
    //ENDPOINT COMPLETO
    $endpoint = $this->baseUrl.$resource;
    
    //HEADERS
    $headers = [
      'Content-type: application/json',
      'Authorization: Bearer '.$this->getAccessToken()
    ];
    
    
    //CONFIGURAÇÃO DO CURL
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => $endpoint,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => '',
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST  => $method,
      CURLOPT_HTTPHEADER => array(
        'Authorization: Bearer '.$this->getAccessToken(),
        'User-Agent: insomnia/2023.5.5'
      ),
    ));
    
    switch ($method) {
      case 'POST':
      case 'PUT':
      case 'PATCH':
        curl_setopt($curl,CURLOPT_POSTFIELDS,json_encode($request));
        break;
    }

    //EXECUTA O CURL
    $response = curl_exec($curl);
    curl_close($curl);
    
    //RESPONSE EM ARRAY
    $responseArray = json_decode($response,true);
    
    //RETORNA O ARRAY DA RESPOSTA
    return $responseArray;
  }

}