<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Boletreceber;
use Geral\Entities\Banco;
use Geral\Entities\Endeclie;
use Geral\Entities\Contcorr;
use Financeiro\Util\Util;
use Geral\Entities\Empresa;
use Teknisa\Libs\Util\DateUtil;
use Geral\Entities\Licencauso;
use Geral\Entities\Cliente;
use Geral\Entities\Filial;
use Geral\Entities\Municipio;

use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;

class GeraCnabItauService extends GeraCnabService{

    const IDENTIFICACAO_REGISTRO = '0';
    const IDENTIFICACAO_FITA_REMESSA = '1';
    const IDENTIFICACAO_MOVIMENTO = 'REMESSA';
    const IDENTIFICACAO_TIPO_SERVICO = '01';
    const IDENTIFICACAO_POR_EXTENSO_TIPO_SERVICO = 'COBRANCA';
    const IDENTIFICACAO_REGISTRO_TRANSACAO = '1';
    const TIPO_INSCRICAO_EMPRESA_CPF = '01';
    const TIPO_INSCRICAO_EMPRESA_CGC = '02';
    const CODIGO_BANCO = '341';
    const CODIGO_CARTEIRA = 'I';
    const IDENTIFICACAO_REGISTRO_TRAILLER = '9';

    public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL, $idtipcnabrec = NULL, $contreg = NULL, $cdfilial, $contador = NULL){
        try {
            if (!is_dir($caminho))
                mkdir($caminho, "0777", true);
            $nmarquivo = $caminho.$nmarquivo;
            $file = fopen($nmarquivo, "w+");

            // CAMPO: Identificação do Registro
            // POSIÇÃO: 001 a 001
            // PICTURE: 9(1)
            $arquivo = self::IDENTIFICACAO_REGISTRO;

            // CAMPO: Identificação da fita remessa
            // POSIÇÃO: 002 a 002
            // PICTURE: 9(1)
            $arquivo .= self::IDENTIFICACAO_FITA_REMESSA;

            // CAMPO: Identificação por extenso do movimento
            // POSIÇÃO: 003 a 009
            // PICTURE: X(7)
            $arquivo .= self::IDENTIFICACAO_MOVIMENTO;

            // CAMPO: Identificação do tipo de serviço
            // POSIÇÃO: 010 a 011
            // PICTURE: 9(2)
            $arquivo .= self::IDENTIFICACAO_TIPO_SERVICO;

            // CAMPO: Identificação por extenso do tipo de serviço
            // POSIÇÃO: 012 a 026
            // PICTURE: X(15)
            $arquivo .= str_pad(self::IDENTIFICACAO_POR_EXTENSO_TIPO_SERVICO, 15, ' ');

            //************************************************************************
            // CAMPO: Identificação da empresa no banco
            // POSIÇÃO: 027 a 038
            // PICTURE: 9(12)
            /** @var $contcorr Contcorr */
            $contcorr = Contcorr::findOneByCdreducont($cdreducont);
            //Nº da agência do Itaú em que o cliente opera:
            if (!$contcorr && !$contcorr->getCdagencia())
                $arquivo = str_pad($arquivo, 30, '0');
            else{
                $arquivo .= Util::campoCnabValido(substr($contcorr->getCdagencia(), 0, 4));
            }


            //Zeros, complementos do campo - Picture: 9(2)
            $arquivo .= '00';

            if (!$contcorr && !$contcorr->getCdcontcorr())
                $arquivo = str_pad($arquivo, 38, '0');
            else
                $arquivo .= Util::campoCnabValido($contcorr->getCdcontcorr());
            //************************************************************************

            // CAMPO: Brancos
            // POSIÇÃO: 039 a 046
            // PICTURE: X(8)
            $arquivo = str_pad($arquivo, 46, ' ');

            // CAMPO: Nome da Empresa
            // POSIÇÃO: 047 a 076
            // PICTURE: X(30)
            /** @var $empresa Empresa */
            $empresa = Empresa::findOneByCdempresa($cdempresa);
    
            if($empresa){
                if($cdfilial){
                   $filial = Filial::findOneByCdfilial($cdfilial); 
                   if($filial){
                       $nmempresa = substr(Util::campoCnabValido($filial->getNmrazsocfili()), 0, 30);
                   } else {
                      $nmempresa = substr(Util::campoCnabValido($empresa->getNmempresa()), 0, 30); 
                   }
                   
                   $arquivo .= str_pad($nmempresa, 30);
                   
                } else {
                    $nmempresa = substr(Util::campoCnabValido($empresa->getNmempresa()), 0, 30);
                    $arquivo .= str_pad($nmempresa, 30);
                }
            }

            // CAMPO: Código do Banco
            // POSIÇÃO: 077 a 079
            // PICTURE: 9(3)
            $arquivo .= self::CODIGO_BANCO;

            // CAMPO: Nome do Banco
            // POSIÇÃO: 080 a 094
            // PICTURE: X(15)
            if($contcorr){
                /** @var $banco Banco */
                $banco = Banco::findOneByCdbanco($contcorr->getCdbanco());
                if($banco){
                    $nmbanco = substr(Util::campoCnabValido($banco->getNmbanco()), 0, 15);
                    $arquivo .= str_pad($nmbanco, 15);
                }
            }

            // CAMPO: Data de Gravação da Fita
            // POSIÇÃO: 095 a 100
            // PICTURE: 9(6)
            $now = DateUtil::getDataAtual();
            $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
            $arquivo .= Util::campoCnabValido($data);

            // CAMPO: Brancos
            // POSIÇÃO: 101 a 394
            // PICTURE: X(294)
            $arquivo = str_pad($arquivo, 394);

            //Valida se a inscrição da empresa possuir mais de 14 Caracteres
            if($empresa && strlen($empresa->getNrinsjurempr()) > 14)
                throw new \Exception('A inscrição da empresa é inválida');

            // CAMPO: Número Seqüencial
            // POSIÇÃO: 395 a 400
            // PICTURE: 9(6)
            $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT) . "\r\n";

            //Grava a variavel arquivo no arquivo texto
            fprintf($file, "%s", $arquivo);
            fclose($file);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
                                  $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
                                  $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
                                  $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL, $cdreducont,$cdbanco)
    {
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRANSACAO;

        // CAMPO: Tipo da inscrição da empresa/sacador
        // POSIÇÃO: 002 a 003
        // PICTURE: 9(2)
        /** @var $empresa Empresa */
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        
        if($empresa && $empresa->getIdtpijurempr() == 'F')
            $arquivo .= self::TIPO_INSCRICAO_EMPRESA_CPF;
        else
            $arquivo .= self::TIPO_INSCRICAO_EMPRESA_CGC;

        // CAMPO: Nº de inscrição da empresa/sacador
        // POSIÇÃO: 004 a 017
        // PICTURE: 9(14)
         /**
            * Caso seja informado filial na geração de remessa, sera passado o número de inscrição CNPJ da filial no lugar da empresa.
        */
        if($empresa){
            if($idfilialcnabrec){
                $filial = Filial::findOneByCdfilial($idfilialcnabrec);
                $arquivo .= str_pad($filial->getNrinsjurfili(), 14, '0', STR_PAD_LEFT);
            } else {
                $arquivo .= str_pad($empresa->getNrinsjurempr(), 14, '0', STR_PAD_LEFT);
            }
        }
            

        //******************************************************************
        // CAMPO: Identificação da empresa no banco
        // POSIÇÃO: 018 a 029
        // PICTURE: 9(12)
        //Nº da agência do Itaú em que o cliente opera:
        if (!$cdagencia)
            $arquivo = str_pad($arquivo, 21, '0');
        else
            $arquivo .= Util::campoCnabValido(substr($cdagencia, 0, 4));

        //Zeros, complementos do campo - Picture: 9(2)
        $arquivo .= '00';

        if (!$cdcontcorr)
            $arquivo = str_pad($arquivo, 29, '0');
        else
            $arquivo .= Util::campoCnabValido($cdcontcorr);
        // CAMPO: Identificação da empresa no banco
        // POSIÇÃO: 018 a 029
        // PICTURE: 9(12)
        //**************************************************************//

        // CAMPO: Brancos
        // POSIÇÃO: 030 a 033
        // PICTURE: X(4)
        $arquivo = str_pad($arquivo, 33);

        // CAMPO: Instrução / Alegação
        // POSIÇÃO: 034 a 037
        // PICTURE: 9(4)
        $arquivo = str_pad($arquivo, 37, '0');

        // CAMPO: Identificação do título na empresa
        // POSIÇÃO: 038 a 062
        // PICTURE: X(25)
        $arquivo .= str_pad($cdfilial . $nrlanbolrec, 25);

        // CAMPO: Identificação do título no banco
        // POSIÇÃO: 063 a 070
        // PICTURE: 9(8)
        if($idimprboleto && $idimprboleto == 'E'){
            if(!$cdnosnumrec)
                $arquivo = str_pad($arquivo, 70, '0');
            else
                $arquivo .= str_pad(Util::campoCnabValido(substr($cdnosnumrec,0,8)),8,'0',STR_PAD_LEFT);
        }
        else
            $arquivo = str_pad($arquivo, 70, '0');

        // CAMPO: Quantidade de moeda variável
        // POSIÇÃO: 071 a 083
        // PICTURE: 9(8)V9(5)
        $arquivo = str_pad($arquivo, 83, '0');

        // CAMPO: Nº da carteira
        // POSIÇÃO: 084 a 086
        // PICTURE: 9(3)
        if(!$cdcartcobbol)
            $arquivo = str_pad($arquivo, 86, '0');
        else
            $arquivo .= str_pad($cdcartcobbol, 3, '0', STR_PAD_LEFT);

        // CAMPO: Identificação da operação no banco
        // POSIÇÃO: 087 a 107
        // PICTURE: X(21)
        $arquivo = str_pad($arquivo, 107);

        // CAMPO: Código da Carteira
        // POSIÇÃO: 108 a 108
        // PICTURE: 9(1)
        $arquivo .= self::CODIGO_CARTEIRA;

        // CAMPO: Identificação da ocorrência
        // POSIÇÃO: 109 a 110
        // PICTURE: 9(2)
        if($idopecnabrec && $idopecnabrec == 'E')
            $arquivo .= '02';    //Exclusão
        elseif($idopecnabrec == 'I')
            $arquivo .= '01';    //Inclusão
        else
            $arquivo .= '06';    //Alteração

        // CAMPO: Nº do documento de cobrança
        // POSIÇÃO: 111 a 120
        // PICTURE: X(10)
        /** @var $entitieBoleto \Financeiro\Repositories\Boletreceber */
        $entitieBoleto = Boletreceber::getRepository();
        $duplicata = $entitieBoleto->getDuplicata($cdfilial, $nrlanbolrec);
        if($duplicata){
            $arquivo .= substr($duplicata[0]['DUPLICATA'], -7) . '/' . $duplicata[0]['NRPARCELA'];
        }
        else
            $arquivo = str_pad($arquivo, 120);

        // CAMPO: Data de vencimento do título
        // POSIÇÃO: 121 a 126
        // PICTURE: 9(6)
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $data = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . substr(DateUtil::getAno($vencimento), -2);
        $arquivo .= Util::campoCnabValido($data);

        // CAMPO: Valor do título
        // POSIÇÃO: 127 a 139
        // PICTURE: 9(11)V9(2)
        $vr = $vrfinalbrec * 100;
        $vr = str_pad($vr, 13, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vr);

        // CAMPO: Código do banco
        // POSIÇÃO: 140 a 142
        // PICTURE: 9(3)
        $arquivo .= self::CODIGO_BANCO;

        // CAMPO: Agência cobradora
        // POSIÇÃO: 143 a 147
        // PICTURE: 9(5)
        $arquivo = str_pad($arquivo, 147, '0');

        // CAMPO: Espécie do título
        // POSIÇÃO: 148 a 149
        // PICTURE: 9(2)
        $arquivo .= '01'; //????

        // CAMPO: Identificação de título aceito ou não aceito
        // POSIÇÃO: 150 a 150
        // PICTURE: X(1)
        $arquivo .= 'N';

        // CAMPO: Data de emissão do título
        // POSIÇÃO: 151 a 156
        // PICTURE: 9(6)
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
        $arquivo .= Util::campoCnabValido($data);

        // CAMPO: Instrução 1
        // POSIÇÃO: 157 a 158
        // PICTURE: 9(2)
        if($cdins1cobbol)
            $arquivo .= $cdins1cobbol;
        else
            $arquivo = str_pad($arquivo, 158);

        // CAMPO: Instrução 2
        // POSIÇÃO: 159 a 160
        // PICTURE: 9(2)
        if($cdins2cobbol)
            $arquivo .= $cdins2cobbol;
        else
            $arquivo = str_pad($arquivo, 160);

        // CAMPO: Valor de mora por dia de atraso
        // POSIÇÃO: 161 a 173
        // PICTURE: 9(11)V9(2)
        if((($vrpemobrec == 0) or ((bool) $vrpemobrec == false)) and (($vrmordiabrec == 0) or ((bool) $vrmordiabrec == false))){
            $arquivo = str_pad($arquivo, 173, '0');
        } else {
            if(($vrpemobrec != 0) or ((bool) $vrpemobrec == true)){
                $arquivo .= str_pad(str_replace('.', '', $vrpemobrec * 100), 13, '0', STR_PAD_LEFT);
            } else {
                $arquivo .= str_pad(str_replace('.', '', $vrmordiabrec * 100), 13, '0', STR_PAD_LEFT);
            }
        }

        // CAMPO: Data limite p/ concessão de desconto
        // POSIÇÃO: 174 a 179
        // PICTURE: 9(6)
        $arquivo = str_pad($arquivo, 179, '0');

        // CAMPO: Valor do Desconto
        // POSIÇÃO: 180 a 192
        // PICTURE: 9(11)V9(2)
        $arquivo = str_pad($arquivo, 192, '0');

        // CAMPO: Valor do I.O.F.
        // POSIÇÃO: 193 a 205
        // PICTURE: 9(11)V9(2)
        $arquivo = str_pad($arquivo, 205, '0');

        // CAMPO: Valor do Abatimento a ser concedido
        // POSIÇÃO: 206 a 218
        // PICTURE: 9(11)V9(2)
        $arquivo = str_pad($arquivo, 218, '0');

        // ###############################################
        // VERIFICA LICENÇA DE USO DO CLIENTE  ###########
        // ###############################################
        $nrInsJur = 0;
        $nrCepClie = 0;
        $nrInsJur = str_pad($nrInsJur, 14, '0');
        $nrCepClie = str_pad($nrCepClie, 8, '0');
        /** @var $entitieLincenca \Geral\Repositories\Licencauso */
        //$entitieLincenca = Licencauso::getRepository();
        $licenca = null;//$entitieLincenca->getLicencaUso($cdfilial, $nrlanbolrec);
        if ($licenca){
            if ($licenca[0]['IDTPIJURLUSO'] == 'F')
                $tpInscSacado = '01'; //CPF
            else
                $tpInscSacado = '02'; //CPF

            if ($licenca[0]['NRINSJURLUSO'])
                $nrInsJur = $licenca[0]['NRINSJURLUSO'];

            if ($licenca[0]['NRCEPLUSO']){
                $nrCepClie = $licenca[0]['NRCEPLUSO'];
                $nmClie = $licenca[0]['NMFANTLUSO'];
                $dsEndeclie = $licenca[0]['DSENDELUSO'];
                $nmBairro = $licenca[0]['NMBAIRLUSO'];
                $nmMunicipio = $licenca[0]['NMMUNICIPIO'];
                $sgEstado = $licenca[0]['SGESTADO'];
            }
        }
        else {
            /** @var $entitieCliente Cliente */
            $entitieCliente = Cliente::findOneByCdcliente($cdcliente);
            if($entitieCliente){
                if($entitieCliente->getIdtpijurclie() == 'F')
                    $tpInscSacado = '01';
                else
                    $tpInscSacado = '02';

                if ($entitieCliente->getNrinsjurclie())
                    $nrInsJur = $entitieCliente->getNrinsjurclie();
                /** @var $entitieEndeClie Endeclie */
                $entitieEndeClie = Endeclie::findOneByCdclienteAndIdtpendeclie($cdcliente, 'C');
                
                if($entitieEndeClie){
                    if($entitieEndeClie->getNrcepclie())
                        $nrCepClie = $entitieEndeClie->getNrcepclie();
                    $nmClie = $entitieCliente->getNmrazsocclie();
                    $dsEndeclie = $entitieEndeClie->getDsendeclie();
                    $nmBairro = $entitieEndeClie->getNmbairclie();
                    $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $entitieEndeClie->getCdMunicipio());
                    $nmMunicipio = $municipio->getNmmunicipio();
                    $sgEstado = $entitieEndeClie->getSgestado();
                }
            }
        }

        // CAMPO: Código de inscrição/Sacado ('01' = CPF, '02' = CGC)
        // POSIÇÃO: 219 a 220
        // PICTURE: 9(2)
        $arquivo .= $tpInscSacado;

        // CAMPO: Nº de inscrição do sacado (CPF/CGC)
        // POSIÇÃO: 221 a 234
        // PICTURE: 9(14)
        $arquivo .= Util::campoCnabValido(str_pad($nrInsJur, 14, '0', STR_PAD_LEFT));

        // CAMPO: Nome do sacado
        // POSIÇÃO: 235 a 264
        // PICTURE: X(30)
        $nomecliente = substr(Util::campoCnabValido($nmClie), 0, 30);
        $arquivo .= str_pad($nomecliente, 30);

        // CAMPO: Brancos
        // POSIÇÃO: 265 a 274
        // PICTURE: X(10)
        $arquivo = str_pad($arquivo, 274);

        // CAMPO: Logradouro (Endereço do sacado (cliente))
        // POSIÇÃO: 275 a 314
        // PICTURE: X(40)
        $enderecoCliente = substr(Util::campoCnabValido($dsEndeclie), 0, 40);
        $arquivo .= str_pad($enderecoCliente, 40);

        // CAMPO: Bairro do sacado
        // POSIÇÃO: 315 a 326
        // PICTURE: X(12)
        $bairro = substr(Util::campoCnabValido($nmBairro), 0, 12);
        $arquivo .= str_pad($bairro, 12);

        // CAMPO: C.E.P. do sacado
        // POSIÇÃO: 327 a 334
        // PICTURE: 9(8)
        $arquivo .= Util::campoCepTel($nrCepClie);

        // CAMPO: Cidade do sacado
        // POSIÇÃO: 335 a 349
        // PICTURE: X(15)
        $municipio = substr(Util::campoCnabValido($nmMunicipio), 0, 15);
        $arquivo .= str_pad($municipio, 15);

        // CAMPO: Estado do sacado
        // POSIÇÃO: 350 a 351
        // PICTURE: X(2)
        $arquivo .= Util::campoCnabValido(Util::campoCnabValido($sgEstado));

        if($cdins1cobbol == '93' or $cdins1cobbol == '94' or $cdins1cobbol == '94' or $cdins1cobbol == '93'){
            $mensagens = $entitieBoleto->getTxBoleto($cdfilial, $nrlanbolrec);
            $arquivo .= str_pad(substr($mensagens, 0, 30), 30);
        }
        else
            // CAMPO: Nome do sacador ou avalista
            // POSIÇÃO: 352 a 381
            // PICTURE: X(30)
            $arquivo = str_pad($arquivo, 381);

        // CAMPO: Brancos
        // POSIÇÃO: 382 a 385
        // PICTURE: X(4)
        $arquivo = str_pad($arquivo, 385);

        // CAMPO: Data de mora
        // POSIÇÃO: 386 a 391
        // PICTURE: X(6)
        $arquivo = str_pad($arquivo, 391, '0');

        // CAMPO: Prazo (Qtd de dias)
        // POSIÇÃO: 392 a 393
        // PICTURE: 9(2)
        if($qtdiaprotbol)
            $arquivo .= $qtdiaprotbol;
        else
            $arquivo .= '00';

        // CAMPO: Brancos
        // POSIÇÃO: 394 a 394
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 394);

        // CAMPO: Número Seqüencial
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    public function geraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRAILLER;

        // CAMPO: Brancos
        // POSIÇÃO: 002 a 394
        // PICTURE: X(393)
        $arquivo = str_pad($arquivo, 394);

        // CAMPO: Número Seqüencial
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT);

        $arquivo.=  "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial){}
    public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
                                         $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
                                         $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
                                         $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
                                         $qtdiaprotbol, $contador = NULL){}
    public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL){}
    public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL){}
    public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL){}


}