<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Teknisa\Libs\Util\DateUtil;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Model\Geral\Entities\Contcorr;

class CreditoBancarioFactory {
    protected $connection;
    protected $entityManager;

    public function __construct($entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getContaCorrente($cdreducont, $nrorg){
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdreducont'] = $cdreducont;
        $movbancario = $this->entityManager->getRepository(Contcorr::class)->findOneBy($params);
        return $movbancario;
    }

    public function getDataProcessamento($filial, $nrorg){
        $sql = "SELECT DECODE(PG.IDUTILDTPROC, 'S', NVL(PF.DTPROCESSA, SYSDATE), SYSDATE) AS DTPROCESSA
                  FROM PARAMFILIAL PF,
                       PARAMGERAL PG
                 WHERE PF.CDFILIAL = :CDFILIAL
                   AND PF.NRORG = :NRORG
                   AND ROWNUM      = 1";
        $params = array(
            "CDFILIAL"=>$filial,
            "NRORG"=>$nrorg);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function getSaldoDia($filial, $nrorg, $dataFiliProc){
        $sql = "SELECT VRSDOCCORR
                  FROM SDOCCORR
                 WHERE CDREDUCONT = :CDREDUCONT
                   AND NRORG      = :NRORG
                   AND DTSDOCCORR = (SELECT MAX(DTSDOCCORR)
                                       FROM SDOCCORR
                                      WHERE CDREDUCONT =  :CDREDUCONT
                                        AND NRORG      =  :NRORG
                                        AND DTSDOCCORR <= :DATAFILIPROC)";
        $params = array(
            "CDFILIAL"     => $filial,
            "NRORG"        => $nrorg,
            "DATAFILIPROC" => $dataFiliProc);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function updateContaCorrente(Contcorr $contcorr, $valorCredito){
        $contcorr->setVrutlcobdesc($contcorr->getVrutlcobdesc() + $valorCredito);
        $contcorr->setDtinclusao(DateUtil::getDataAtual());
        return $contcorr;
    }

}