<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;
use SimpleXMLElement;

class OfxService {

    private $xmlOfx;

    private function calculaSandoExtrado($movs) {
        $saldo = 0;
        for ($i=0; $i < count($movs) ; $i++) {
            if ($movs[$i]['IDMOVREAL'] === 'D') {
                $movs[$i]['VRMOVREAL'] = $movs[$i]['VRMOVREAL'] * (-1);
            }
            $saldo = $saldo + $movs[$i]['VRMOVREAL'];
        }
        return $saldo;
    }

    private function getBanco() {
        return trim($this->xmlOfx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKACCTFROM->BANKID->__toString());
    }

    private function getConta() {
        $contaCorrente = trim($this->xmlOfx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKACCTFROM->ACCTID->__toString());
        $contaCorrente = str_replace('-X', '-0', $contaCorrente);
        $contaCorrente = str_replace('-', '', $contaCorrente);
        return $contaCorrente;
    }

    private function getDtFin() {
        $data =  $this->xmlOfx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKTRANLIST->DTEND;
        return substr($data, 6, 2).'/'.substr($data, 4, 2).'/'.substr($data, 0, 4);
    }

    private function getDtInic() {
        $data =  $this->xmlOfx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKTRANLIST->DTSTART;
        return substr($data, 6, 2).'/'.substr($data, 4, 2).'/'.substr($data, 0, 4);
    }

    private function getVrfin() {
        $vrFin = $this->xmlOfx->BANKMSGSRSV1->STMTTRNRS->STMTRS->LEDGERBAL->BALAMT;
        if ($vrFin) {
            return floatval($vrFin);
        } else {
            return 0;
        }
    }

    private function getMovs() {
        $movExtrato = $this->xmlOfx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKTRANLIST->STMTTRN;
        $lenghtMovs =  count($movExtrato);
        $movs = array();
        for ($i=0; $i < $lenghtMovs ; $i++) {
            $mov = $movExtrato[$i];
            $idmovreal   = substr($mov->TRNTYPE, 0, 1);
            $dtMovreal   = substr($mov->DTPOSTED, 6, 2).'/'.substr($mov->DTPOSTED, 4, 2).'/'.substr($mov->DTPOSTED, 0, 4);
            $vrMovreal   = $this->formatMoney($mov->TRNAMT);
            $cdFitIdExtr = str_pad(trim($mov->FITID), 20, '0', STR_PAD_LEFT);

            if($vrMovreal < 0) {
                $idmovreal = 'D';
                $vrMovreal = $vrMovreal * (-1);
            } else {
                $idmovreal = 'C';
            }
            
            $nrdocmovreal = str_pad(trim($movExtrato[$i]->CHECKNUM), 20, '0', STR_PAD_LEFT);
            
            $movs[] = array ('IDMOVREAL' => $idmovreal,
                            'DTMOVREAL' => $dtMovreal,
                            'VRMOVREAL' => $vrMovreal,
                            'CDFITIDEXTR' => $cdFitIdExtr,
                            'NRDOCMOVREAL' => $nrdocmovreal,
                            'DSHISTMOVREAL' => trim($movExtrato[$i]->MEMO->__toString()),
                            'IDTPMOVREAL' => 0,
                            'IDSTCONMVBAN' => 0,
                            'IDTIPOREGI' => 0
                            );
        }
        return $movs;
    }

    private function formatMoney($value)
    {
        $value = trim($value);
        $virgula = strpos($value, ',');
        if ($virgula){
            $value = str_replace(".", "", $value);
            $value = str_replace(",", ".", $value);
            $value = floatval($value);
        }else{
            $pont = substr($value,-3, 1);
            if ($pont == '.')
                $value = substr_replace($value, ',', -3, 1);
            $value = str_replace(".", "", $value);
            $value = str_replace(",", ".", $value);
            $value = floatval($value);
        }
        return $value;
    }

    private function getVrInic($movs, $saldoFin) {
        $saldoExtrato = $this->calculaSandoExtrado($movs);

        return $saldoFin + $saldoExtrato;
        
    }

 /*   private function getOfxAsXML($content) {
        $line = strpos($content, "<OFX>");
        $ofx = substr($content, $line - 1);
        $buffer = $ofx;        $count = 0;
        while ($pos = strpos($buffer, '<')) {
            $count++;
            $pos2 = strpos($buffer, '>');
            $element = substr($buffer, $pos + 1, $pos2 - $pos - 1);
            if (substr($element, 0, 1) == '/'){
                $sla[] = substr($element, 1);
            } 
            else {
                $als[] = $element;
            }
            $buffer = substr($buffer, $pos2 + 1);
        }
        if(!isset($als)){
            $als = [];
        }
        if(!isset($sla)){
            $sla = [];
        }
        $adif = array_diff($als, $sla);
        $adif = array_unique($adif);
        $ofxy = $ofx;
        foreach ($adif as $dif) {
            $dpos = 0;
            while ($dpos = strpos($ofxy, "<$dif>", $dpos + 1)) {
                $npos = strpos($ofxy, '<', $dpos + 1);
                $ofxy = substr_replace($ofxy, "</$dif>\n<", $npos, 1);
                $dpos = $npos + strlen($element) + 3;
            }
        }
        $ofxy = str_replace('&', '&', $ofxy);
        return $ofxy;
    }

*/
     
    private function getOfxAsXML($content) {
        $line = strpos($content, "<OFX>");
        $ofx = substr($content, $line - 1);
        $buffer = $ofx;        
        $count = 0;
        
        // INICIO - MODIFICADO SICREDI
        while (strpos($buffer, '<') > -1) {
            $pos = strpos($buffer, '<');
            // FIM - MODIFICADO SICREDI
            
            $count++;
            $pos2 = strpos($buffer, '>');
            $element = substr($buffer, $pos + 1, $pos2 - $pos - 1);
            // Chamada da função para remover todos os caracteres especiais das tags <MEMO> <FITID> <REFNUM>
            $ofx = $this->removeOriginalStringCharacter($buffer,$element, $pos2, $ofx);
            
            if (substr($element, 0, 1) == '/'){
                $sla[] = substr($element, 1);
            } 
            else {
                $als[] = $element;
            }
            
            $buffer = substr($buffer, $pos2 + 1);
        }
        if(!isset($als)){
            $als = [];
        }
        if(!isset($sla)){
            $sla = [];
        }
        
        foreach ($sla as $item) {
            $key = array_search($item, $als);
            if ( $key !== false ) {
                unset($als[$key]);
            }
        }
        
        /* ########### INICIO ALTERAÇÃO VICTOR - PARA BANCO BB ######## */
        //$adif = array_diff($als, $sla);
        //$adif = array_unique($adif);
        
        $adif = $als;
        /* ########### FIM ALTERAÇÃO VICTOR - PARA BANCO BB ######## */
        
        $ofxy = $ofx;
        foreach ($adif as $dif) {
            $dpos = 0;
            while ($dpos = strpos($ofxy, "<$dif>", $dpos + 1)) {
                $npos = strpos($ofxy, '<', $dpos + 1);
                
                /* ########### INICIO ALTERAÇÃO VICTOR - PARA BANCO BB ######## */
                $tamanho = (strlen($dif)+3);
                $posicao = intval($npos)-(strlen($dif)+3);
                $tag = substr ($ofxy  , $npos, $tamanho);
                $montada = '</'.$dif.'>';
                
                if($tag != $montada){
                     $ofxy = substr_replace($ofxy, "</$dif>\n<", $npos, 1);
                }
                /* ########### FIM ALTERAÇÃO VICTOR - PARA BANCO BB ######## */
                    
                $dpos = $npos + strlen($element) + 3;
            }
        }

        return $ofxy;
    }
        
        
        

    private function OfxAsObject($stringOfx) {
        $stringOfx = utf8_encode($stringOfx);
        $xmlOfx = $this->getOfxAsXML($stringOfx);
        $objectOfx = simplexml_load_string($xmlOfx);

        return $objectOfx;
    }
    

    public function readOfx($ofx) {
        $this->xmlOfx = $this->OfxAsObject($ofx);

        $banco = $this->getBanco();
        $conta = $this->getConta();
        $dtInic = $this->getDtInic();
        $dtFin = $this->getDtFin();
        $vrFin = $this->getVrfin();

        if ($vrFin < 0) {
            $vrFin = $vrFin * (-1);
            $idextrfim = 'D';
        } else {
            $idextrfim = 'C';
        }
        $movimentacoes = $this->getMovs();
        
        if($vrFin === 0) {
            $vrInic = 0;
            $idextrini = 'C';
        } else {
            $vrInic = $this->getVrInic($movimentacoes, $vrFin);
            if ($vrInic < 0) {
                $vrInic = $vrInic * (-1);
                $idextrini = 'D';
            } else {
                $idextrini = 'C';
            }
        }
        $extrato = array('BANCO' => $banco, //Banco
                        'CONTA' => $conta, //Conta
                        'IDEXTRINI' => $idextrini, //Indentifacod do saldo Inicial
                        'DTSALDINIC' => $dtInic, //Data Incial
                        'VRSALDINIC' => $vrInic, //Valor Inicial
                        'IDEXTRFIM' => $idextrfim, //Identificador do saldo final
                        'DTSALDFIN' => $dtFin, //Data Final
                        'VRSALDFIM' => $vrFin, //Valor Final
                        'MOVS' => $movimentacoes, //Movimentações
                         );
                         
        return $extrato;
    }
    
    /**
     * Remove todos os caracteres especiais que estão nas tags <MEMO> <FITID> <REFNUM>
     * @param $buffer -> String que contem os dados do arquivo
     * @param $element -> Tags sem os indentificadores <>
     * @param $pos2 -> Posição do identificados >
     * @param $ofx -> String Original
     * 
    */
    public function removeOriginalStringCharacter($buffer,$element, $pos2, $ofx) {
        $fin = strpos($buffer, '</');
        if($element == "FITID"){
            $contentTag = substr($buffer, $pos2 + 1, $fin - $pos2 - 1);
            $removeStringCaractere = preg_replace('/[^\p{L}0-9<>_\s]/u',' ',$contentTag);
            $ofx = str_replace($contentTag, $removeStringCaractere, $ofx);
        }
        
        if($element == "MEMO"){
            $contentTag = substr($buffer, $pos2 + 1, $fin - $pos2 - 1);
            $removeStringCaractere = preg_replace('/[^\p{L}0-9<>_\s]/u',' ',$contentTag);
            $ofx = str_replace($contentTag, $removeStringCaractere, $ofx);
        }
        
        if($element == "REFNUM"){
            $contentTag = substr($buffer, $pos2 + 1, $fin - $pos2 - 1);
            $removeStringCaractere = preg_replace('/[^\p{L}0-9<>_\s]/u',' ',$contentTag);
            $ofx = str_replace($contentTag, $removeStringCaractere, $ofx);
        }
        
        if($element == "NAME"){
            $contentTag = substr($buffer, $pos2 + 1, $fin - $pos2 - 1);
            $removeStringCaractere = preg_replace('/[^\p{L}0-9<>_\s]/u',' ',$contentTag);
            $ofx = str_replace($contentTag, $removeStringCaractere, $ofx);
        }
        return $ofx;
    }
    
        function closetag($ofx) {
        $xml    = new SimpleXMLElement($ofx);
        $domxml = dom_import_simplexml($xml);
        return $dom->saveXML();
        }

}